@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1997 Vectaport Inc.
+ * Copyright (c) 2000 IET Inc.
+ * Copyright (c) 1997,1999 Vectaport Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and
  * its documentation for any purpose is hereby granted without fee, provided
@@ -32,37 +33,53 @@
 
 /*****************************************************************************/
 
+int AssignFunc::_symid = -1;
+
 AssignFunc::AssignFunc(ComTerp* comterp) : ComFunc(comterp) {
 }
 
 
 void AssignFunc::execute() {
     ComValue operand1(stack_arg(0, true));
-    ComValue* operand2 = new ComValue(stack_arg(1));
+    if (operand1.type() != ComValue::SymbolType) {
+      operand1.assignval(stack_arg_post_eval(0, true /* no symbol lookup */));
+    }
+    ComValue* operand2 = new ComValue(stack_arg_post_eval(1, true /* no symbol lookup */));
+    if (operand2->is_attribute()) lookup_symval(*operand2);
     reset_stack();
     if (operand1.type() == ComValue::SymbolType) {
         AttributeList* attrlist = comterp()->get_attributes();
 	if (attrlist) {
 	    Resource::ref(attrlist);
 	    Attribute* attr = new Attribute(operand1.symbol_val(), 
-					    new AttributeValue(*operand2));
+					    operand2);
 	    attrlist->add_attribute(attr);
 	    Unref(attrlist);
-	} else 
+	} else if (operand1.global_flag()) 
+	    comterp()->globaltable()->insert(operand1.symbol_val(), operand2);
+	else
 	    comterp()->localtable()->insert(operand1.symbol_val(), operand2);
+    } else if (operand1.is_object(Attribute::class_symid())) {
+      Attribute* attr = (Attribute*)operand1.obj_val();
+      attr->Value(operand2);
     } else {
-        cerr << "assignment to something other than a symbol ignored\n";
+        cerr << "assignment to something other than a symbol or attribute ignored\n";
+	delete operand2;
     }
     push_stack(*operand2);
 }
 
-ModAssignFunc::ModAssignFunc(ComTerp* comterp) : ModFunc(comterp) {
+ModAssignFunc::ModAssignFunc(ComTerp* comterp) : AssignFunc(comterp) {
 }
 
 
 void ModAssignFunc::execute() {
     ComValue operand1(stack_arg(0, true));
-    ComValue operand2(stack_arg(1));
+    if (operand1.type() != ComValue::SymbolType) {
+      operand1.assignval(stack_arg_post_eval(0, true /* no symbol lookup */));
+    }
+    ComValue operand2(stack_arg_post_eval(1, true /* no symbol lookup */));
+    if (operand2.is_attribute()) lookup_symval(operand2);
     reset_stack();
     if (operand1.type() == ComValue::SymbolType) {
         void* op1val = nil;
@@ -74,9 +91,8 @@ void ModAssignFunc::execute() {
 	push_stack(*(ComValue*)op1val);
 	delete (ComValue*)op1val;
 	push_stack(operand2);
-        push_funcstate(2,0);
-	ModFunc::execute();
-	pop_funcstate();
+	static ModFunc* subfunc = new ModFunc(comterp());
+	subfunc->exec(2,0);
 	ComValue* result = new ComValue(pop_stack());
         _comterp->localtable()->insert(operand1.symbol_val(), result);
 	push_stack(*result);
@@ -84,13 +100,17 @@ void ModAssignFunc::execute() {
 
 }
 
-MpyAssignFunc::MpyAssignFunc(ComTerp* comterp) : MpyFunc(comterp) {
+MpyAssignFunc::MpyAssignFunc(ComTerp* comterp) : AssignFunc(comterp) {
 }
 
 
 void MpyAssignFunc::execute() {
     ComValue operand1(stack_arg(0, true));
-    ComValue operand2(stack_arg(1));
+    if (operand1.type() != ComValue::SymbolType) {
+      operand1.assignval(stack_arg_post_eval(0, true /* no symbol lookup */));
+    }
+    ComValue operand2(stack_arg_post_eval(1, true /* no symbol lookup */));
+    if (operand2.is_attribute()) lookup_symval(operand2);
     reset_stack();
     if (operand1.type() == ComValue::SymbolType) {
         void* op1val = nil;
@@ -102,9 +122,8 @@ void MpyAssignFunc::execute() {
 	push_stack(*(ComValue*)op1val);
 	delete (ComValue*)op1val;
 	push_stack(operand2);
-        push_funcstate(2,0);
-	MpyFunc::execute();
-	pop_funcstate();
+	static MpyFunc* subfunc = new MpyFunc(comterp());
+	subfunc->exec(2,0);
 	ComValue* result = new ComValue(pop_stack());
         _comterp->localtable()->insert(operand1.symbol_val(), result);
 	push_stack(*result);
@@ -112,13 +131,17 @@ void MpyAssignFunc::execute() {
 
 }
 
-AddAssignFunc::AddAssignFunc(ComTerp* comterp) : AddFunc(comterp) {
+AddAssignFunc::AddAssignFunc(ComTerp* comterp) : AssignFunc(comterp) {
 }
 
 
 void AddAssignFunc::execute() {
     ComValue operand1(stack_arg(0, true));
-    ComValue operand2(stack_arg(1));
+    if (operand1.type() != ComValue::SymbolType) {
+      operand1.assignval(stack_arg_post_eval(0, true /* no symbol lookup */));
+    }
+    ComValue operand2(stack_arg_post_eval(1, true /* no symbol lookup */));
+    if (operand2.is_attribute()) lookup_symval(operand2);
     reset_stack();
     if (operand1.type() == ComValue::SymbolType) {
         void* op1val = nil;
@@ -130,9 +153,8 @@ void AddAssignFunc::execute() {
 	push_stack(*(ComValue*)op1val);
 	delete (ComValue*)op1val;
 	push_stack(operand2);
-        push_funcstate(2,0);
-	AddFunc::execute();
-	pop_funcstate();
+	static AddFunc* subfunc = new AddFunc(comterp());
+	subfunc->exec(2,0);
 	ComValue* result = new ComValue(pop_stack());
         _comterp->localtable()->insert(operand1.symbol_val(), result);
 	push_stack(*result);
@@ -140,13 +162,17 @@ void AddAssignFunc::execute() {
 
 }
 
-SubAssignFunc::SubAssignFunc(ComTerp* comterp) : SubFunc(comterp) {
+SubAssignFunc::SubAssignFunc(ComTerp* comterp) : AssignFunc(comterp) {
 }
 
 
 void SubAssignFunc::execute() {
     ComValue operand1(stack_arg(0, true));
-    ComValue operand2(stack_arg(1));
+    if (operand1.type() != ComValue::SymbolType) {
+      operand1.assignval(stack_arg_post_eval(0, true /* no symbol lookup */));
+    }
+    ComValue operand2(stack_arg_post_eval(1, true /* no symbol lookup */));
+    if (operand2.is_attribute()) lookup_symval(operand2);
     reset_stack();
     if (operand1.type() == ComValue::SymbolType) {
         void* op1val = nil;
@@ -158,9 +184,8 @@ void SubAssignFunc::execute() {
 	push_stack(*(ComValue*)op1val);
 	delete (ComValue*)op1val;
 	push_stack(operand2);
-        push_funcstate(2,0);
-	SubFunc::execute();
-        pop_funcstate();
+	static SubFunc* subfunc = new SubFunc(comterp());
+	subfunc->exec(2,0);
 	ComValue* result = new ComValue(pop_stack());
         _comterp->localtable()->insert(operand1.symbol_val(), result);
 	push_stack(*result);
@@ -168,13 +193,17 @@ void SubAssignFunc::execute() {
 
 }
 
-DivAssignFunc::DivAssignFunc(ComTerp* comterp) : DivFunc(comterp) {
+DivAssignFunc::DivAssignFunc(ComTerp* comterp) : AssignFunc(comterp) {
 }
 
 
 void DivAssignFunc::execute() {
     ComValue operand1(stack_arg(0, true));
-    ComValue operand2(stack_arg(1));
+    if (operand1.type() != ComValue::SymbolType) {
+      operand1.assignval(stack_arg_post_eval(0, true /* no symbol lookup */));
+    }
+    ComValue operand2(stack_arg_post_eval(1, true /* no symbol lookup */));
+    if (operand2.is_attribute()) lookup_symval(operand2);
     reset_stack();
     if (operand1.type() == ComValue::SymbolType) {
         void* op1val = nil;
@@ -186,9 +215,8 @@ void DivAssignFunc::execute() {
 	push_stack(*(ComValue*)op1val);
 	delete (ComValue*)op1val;
 	push_stack(operand2);
-        push_funcstate(2,0);
-	DivFunc::execute();
-	pop_funcstate();
+	static DivFunc* subfunc = new DivFunc(comterp());
+	subfunc->exec(2,0);
 	ComValue* result = new ComValue(pop_stack());
         _comterp->localtable()->insert(operand1.symbol_val(), result);
 	push_stack(*result);
@@ -196,11 +224,14 @@ void DivAssignFunc::execute() {
 
 }
 
-IncrFunc::IncrFunc(ComTerp* comterp) : AddFunc(comterp) {
+IncrFunc::IncrFunc(ComTerp* comterp) : AssignFunc(comterp) {
 }
 
 void IncrFunc::execute() {
     ComValue operand1(stack_arg(0, true));
+    if (operand1.type() != ComValue::SymbolType) {
+      operand1.assignval(stack_arg_post_eval(0, true /* no symbol lookup */));
+    }
     reset_stack();
     if (operand1.type() == ComValue::SymbolType) {
         void* op1val = nil;
@@ -214,9 +245,8 @@ void IncrFunc::execute() {
 	    one.type(ComValue::IntType);
 	    one.int_ref() = 1;
 	    push_stack(one);
-	    push_funcstate(2,0);
-  	    AddFunc::execute();
-	    pop_funcstate();
+	    static AddFunc* subfunc = new AddFunc(comterp());
+	    subfunc->exec(2,0);
 	    ComValue* result = new ComValue(pop_stack());
             _comterp->localtable()->insert(operand1.symbol_val(), result);
 	    push_stack(*result);
@@ -226,11 +256,14 @@ void IncrFunc::execute() {
 
 }
 
-IncrAfterFunc::IncrAfterFunc(ComTerp* comterp) : AddFunc(comterp) {
+IncrAfterFunc::IncrAfterFunc(ComTerp* comterp) : AssignFunc(comterp) {
 }
 
 void IncrAfterFunc::execute() {
     ComValue operand1(stack_arg(0, true));
+    if (operand1.type() != ComValue::SymbolType) {
+      operand1.assignval(stack_arg_post_eval(0, true /* no symbol lookup */));
+    }
     reset_stack();
     if (operand1.type() == ComValue::SymbolType) {
         void* op1val = nil;
@@ -239,28 +272,30 @@ void IncrAfterFunc::execute() {
 	    push_stack(ComValue::nullval());
 	else {
 	    push_stack(*(ComValue*)op1val);
-	    delete (ComValue*)op1val;
 	    ComValue one;
 	    one.type(ComValue::IntType);
 	    one.int_ref() = 1;
 	    push_stack(one);
-	    push_funcstate(2,0);
-  	    AddFunc::execute();
-	    pop_funcstate();
+	    static AddFunc* subfunc = new AddFunc(comterp());
+	    subfunc->exec(2,0);
 	    ComValue* result = new ComValue(pop_stack());
             _comterp->localtable()->insert(operand1.symbol_val(), result);
 	    push_stack(*(ComValue*)op1val);
+	    delete (ComValue*)op1val;
 	}
     } else 
         push_stack(ComValue::nullval());
 
 }
 
-DecrFunc::DecrFunc(ComTerp* comterp) : SubFunc(comterp) {
+DecrFunc::DecrFunc(ComTerp* comterp) : AssignFunc(comterp) {
 }
 
 void DecrFunc::execute() {
     ComValue operand1(stack_arg(0,true));
+    if (operand1.type() != ComValue::SymbolType) {
+      operand1.assignval(stack_arg_post_eval(0, true /* no symbol lookup */));
+    }
     reset_stack();
     if (operand1.type() == ComValue::SymbolType) {
         void* op1val = nil;
@@ -274,9 +309,8 @@ void DecrFunc::execute() {
 	    one.type(ComValue::IntType);
 	    one.int_ref() = 1;
 	    push_stack(one);
-	    push_funcstate(2,0);
-  	    SubFunc::execute();
-	    pop_funcstate();
+	    static SubFunc* subfunc = new SubFunc(comterp());
+	    subfunc->exec(2,0);
 	    ComValue* result = new ComValue(pop_stack());
             _comterp->localtable()->insert(operand1.symbol_val(), result);
 	    push_stack(*result);
@@ -286,11 +320,14 @@ void DecrFunc::execute() {
 
 }
 
-DecrAfterFunc::DecrAfterFunc(ComTerp* comterp) : SubFunc(comterp) {
+DecrAfterFunc::DecrAfterFunc(ComTerp* comterp) : AssignFunc(comterp) {
 }
 
 void DecrAfterFunc::execute() {
     ComValue operand1(stack_arg(0,true));
+    if (operand1.type() != ComValue::SymbolType) {
+      operand1.assignval(stack_arg_post_eval(0, true /* no symbol lookup */));
+    }
     reset_stack();
     if (operand1.type() == ComValue::SymbolType) {
         void* op1val = nil;
@@ -299,17 +336,16 @@ void DecrAfterFunc::execute() {
 	    push_stack(ComValue::nullval());
 	else {
 	    push_stack(*(ComValue*)op1val);
-	    delete (ComValue*)op1val;
 	    ComValue one;
 	    one.type(ComValue::IntType);
 	    one.int_ref() = 1;
 	    push_stack(one);
-	    push_funcstate(2,0);
-  	    SubFunc::execute();
-	    pop_funcstate();
+	    static SubFunc* subfunc = new SubFunc(comterp());
+	    subfunc->exec(2,0);
 	    ComValue* result = new ComValue(pop_stack());
             _comterp->localtable()->insert(operand1.symbol_val(), result);
 	    push_stack(*(ComValue*)op1val);
+	    delete (ComValue*)op1val;
 	}
     } else 
         push_stack(ComValue::nullval());
