@@ -1,4 +1,6 @@
 /*
+ * Copyright (c) 2001 Scott E. Johnston
+ * Copyright (c) 2000 IET Inc.
  * Copyright (c) 1994-1998 Vectaport Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and
@@ -21,16 +23,22 @@
  * 
  */
 
+#include <ComTerp/comhandler.h>
+
 #include <ComTerp/_comterp.h>
 #include <ComTerp/_comutil.h>
 #include <ComTerp/assignfunc.h>
+#include <ComTerp/bitfunc.h>
 #include <ComTerp/boolfunc.h>
+#include <ComTerp/bquotefunc.h>
 #include <ComTerp/comfunc.h>
-#include <ComTerp/comhandler.h>
 #include <ComTerp/comterp.h>
+#include <ComTerp/comterpserv.h>
 #include <ComTerp/comvalue.h>
 #include <ComTerp/condfunc.h>
 #include <ComTerp/ctrlfunc.h>
+#include <ComTerp/debugfunc.h>
+#include <ComTerp/dotfunc.h>
 #include <ComTerp/helpfunc.h>
 #include <ComTerp/iofunc.h>
 #include <ComTerp/listfunc.h>
@@ -40,16 +48,24 @@
 #include <ComTerp/randfunc.h>
 #include <ComTerp/statfunc.h>
 #include <ComTerp/strmfunc.h>
+#include <ComTerp/symbolfunc.h>
+#include <ComTerp/typefunc.h>
 #include <ComTerp/xformfunc.h>
 #include <Attribute/attrlist.h>
+#include <Attribute/attribute.h>
+#include <OS/math.h>
 
 #include <ctype.h>
 #include <iostream.h>
 #include <string.h>
-#include <strstream.h>
+#include <strstream>
 #include <unistd.h>
+#if __GNUC__>=3
+#include <fstream.h>
+#endif
 
 #define TITLE "ComTerp"
+#define STREAM_MECH
 
 implementTable(ComValueTable,int,void*)
 
@@ -91,6 +107,12 @@ void ComTerp::init() {
     if(dmm_calloc((void**)&_fsstack, _fsstack_siz, sizeof(ComFuncState)) != 0) 
 	KANRET("error in call to dmm_calloc");
 
+    /* Allocate servstate stack to initial size */
+    _ctsstack_top = -1;
+    _ctsstack_siz = 256;
+    if(dmm_calloc((void**)&_ctsstack, _ctsstack_siz, sizeof(ComFuncState)) != 0) 
+	KANRET("error in call to dmm_calloc");
+
     _pfoff = 0;
     _pfnum = 0;
     _quitflag = false;
@@ -99,9 +121,11 @@ void ComTerp::init() {
 
     /* Create ComValue symbol table */
     _localtable = new ComValueTable(100);
-    if (_globaltable) {
+#if 0  /* deferred until first use */
+    if (!_globaltable) {
       _globaltable = new ComValueTable(100);
     }
+#endif
 
     _errbuf = new char[BUFSIZ];
 
@@ -110,6 +134,12 @@ void ComTerp::init() {
     _just_reset = false;
     _defaults_added = false;
     _handler = nil;
+    _val_for_next_func = nil;
+    _func_for_next_expr = nil;
+    _trace_mode = 0;
+    _npause = 0;
+    _stepflag = 0;
+
 }
 
 
@@ -161,36 +191,166 @@ int ComTerp::eval_expr(boolean nested) {
   return FUNCOK;
 }
 
+int ComTerp::eval_expr(ComValue* pfvals, int npfvals) {
+#if 0
+  int save_pfoff = _pfoff;
+  int save_pfnum = _pfnum;
+  ComValue* save_pfcomvals = _pfcomvals;
+#else
+  push_servstate();
+#endif
+
+  _pfoff = 0;
+  _pfnum = npfvals;
+  _pfcomvals = pfvals;
+
+  while (_pfoff < _pfnum) {
+    load_sub_expr();
+    eval_expr_internals();
+  }
+
+#if 0
+  _pfoff = save_pfoff;
+  _pfnum = save_pfnum;
+  _pfcomvals = save_pfcomvals;
+#else
+  pop_servstate();
+#endif
+
+  return FUNCOK;
+}
+
 void ComTerp::eval_expr_internals(int pedepth) {
-  ComValue& sv = pop_stack();
+  static int step_symid = symbol_add("step");
+  static ComFunc* stepfunc = nil;
+  if (!stepfunc)
+    stepfunc = new ComterpStepFunc(this);
+
+  ComValue sv = pop_stack(false);
   
   if (sv.type() == ComValue::CommandType) {
-    
-    ComFunc* func = (ComFunc*)sv.obj_val();
-    func->push_funcstate(sv.narg(), sv.nkey(), pedepth, sv.command_symid());
+
+#ifdef STREAM_MECH
+    /* if func has StreamType ComValue's for arguments */
+    /* create another StreamType ComValue to hold all its */
+    /* arguments, along with a pointer to the func. */
+    boolean has_streams = false;
+    if (!((ComFunc*)sv.obj_val())->post_eval())
+      for(int i=0; i<sv.narg()+sv.nkey(); i++) {
+	if (!stack_top(-i).is_symbol() && !stack_top(-i).is_attribute())
+	  has_streams = stack_top(-i).is_stream();
+	else {
+	  AttributeValue* testval = 
+	    lookup_symval(&stack_top(-i));
+	  has_streams = testval ? testval->is_stream() : false;
+	}
+	if (has_streams) break;
+      }
+    if (has_streams) {
+      AttributeValueList* avl = new AttributeValueList();
+      for(int i=0; i<sv.narg()+sv.nkey(); i++)
+	avl->Prepend(new AttributeValue(pop_stack(true)));
+      ComValue val(sv.obj_val(), avl);
+      val.stream_mode(1); // for external use
+      push_stack(val);
+      return;
+    }
+#endif
+
+    ComFunc* func = nil;
+    if (_func_for_next_expr) {
+      func = _func_for_next_expr;
+      _func_for_next_expr = nil;
+      push_stack(sv);
+      func->push_funcstate(1, 0, pedepth, func->funcid());
+    } else {   
+      func = (ComFunc*)sv.obj_val();
+      func->push_funcstate(sv.narg(), sv.nkey(), 
+			   pedepth, sv.command_symid());
+    }
+
+    /* output execution trace */
+    if (this->trace_mode()) {
+      for(int i=0; i<pedepth; i++) cerr << "    ";
+      cerr << symbol_pntr(sv.command_symid());
+      if (func->post_eval()) 
+	cerr << ": nargs=" << sv.narg() << " nkeys=" << sv.nkey() << "\n";
+      else {
+	int ntotal = func->nargs() + func->nkeys();
+	for(int i=0; i<ntotal; i++) {
+	  if (i) 
+	    cerr << " ";
+	  else 
+	    cerr << "(";
+	  cerr << stack_top(i-ntotal+1);
+	}
+	cerr << ")\n";
+      }
+    }
+
+    if (stepflag()) {
+#if __GNUC__<3
+      filebuf fbufout;
+      fbufout.attach(handler() ? Math::max(1, handler()->get_handle()) : fileno(stdout));
+#else
+      fileptr_filebuf fbufout(handler() && handler()->wrfptr() 
+		      ? handler()->wrfptr() : stdout, 
+		      ios_base::out);
+#endif
+      ostream out(&fbufout);
+      out << ">>> " << *func << "(" << *func->funcstate() << ")\n";
+      static int pause_symid = symbol_add("pause");
+      ComValue pausekey(pause_symid, 0, ComValue::KeywordType);
+      push_stack(pausekey);
+      stepfunc->push_funcstate(0,1, pedepth, step_symid);
+      stepfunc->execute();
+      stepfunc->pop_funcstate();
+      pop_stack();
+    }
+
     func->execute();
     func->pop_funcstate();
-    if (_just_reset) {
+
+    if (_just_reset && !_func_for_next_expr) {
       push_stack(ComValue::blankval());
       _just_reset = false;
     }
     
   } else if (sv.type() == ComValue::SymbolType) {
-    
-    if (_alist) {
-      int id = sv.symbol_val();
-      AttributeValue* val = _alist->find(id);  
-      if (val) {
-	ComValue newval(*val);
-	push_stack(newval);
-      } else
-	push_stack(ComValue::nullval());
-    } else 
+
+    if (_func_for_next_expr) {
+      ComFunc* func = _func_for_next_expr;
+      _func_for_next_expr = nil;
+
       push_stack(sv);
+      func->push_funcstate(1, 0, pedepth, func->funcid());
+      func->execute();
+      func->pop_funcstate();
+      if (_just_reset && val_for_next_func().is_null()) {
+	push_stack(ComValue::blankval());
+	_just_reset = false;
+      }
+
+    } else {
+      
+      if (_alist) {
+	int id = sv.symbol_val();
+	AttributeValue* val = _alist->find(id);  
+	if (val) {
+	  ComValue newval(*val);
+	  push_stack(newval);
+	} else
+	  push_stack(ComValue::nullval());
+      } else 
+	push_stack(lookup_symval(sv));
+    }
+    
+  } else if (sv.is_object(Attribute::class_symid())) {
+    
+    push_stack(*((Attribute*)sv.obj_val())->Value());
     
   } else if (sv.type() == ComValue::BlankType) {
 
-    /* ignore it */
     eval_expr_internals(pedepth);
 
   } else {  /* everything else*/
@@ -211,17 +371,17 @@ void ComTerp::load_sub_expr() {
       token_to_comvalue(_pfbuf+i, sv);
     }
     int offset = 0;
-    for (int i=_pfnum-1; i>=0; i--) {
-      ComValue* sv = _pfcomvals + i;
+    for (int j=_pfnum-1; j>=0; j--) {
+      ComValue* sv = _pfcomvals + j;
       if (sv->is_type(ComValue::CommandType)) {
 	ComFunc* func = (ComFunc*)sv->obj_val();
 	if (func && func->post_eval()) {
 	  int newoffset = offset;
-	  skip_func(_pfcomvals+_pfnum-1, newoffset);
-	  int start = i-1;
+	  skip_func(_pfcomvals+_pfnum-1, newoffset, -_pfnum);
+	  int start = j-1;
 	  int stop = _pfnum+newoffset;
-	  for (int j=start; j>=stop; j--) 
-	    _pfcomvals[j].pedepth()++;
+	  for (int k=start; k>=stop; k--) 
+	    _pfcomvals[k].pedepth()++;
 	}
       }
       offset--;
@@ -242,7 +402,17 @@ void ComTerp::load_sub_expr() {
 	push_stack(argoffval);
       }
     }
-    push_stack(_pfcomvals[_pfoff]);
+    if (!_pfcomvals[_pfoff].is_blank()) {
+      push_stack(_pfcomvals[_pfoff]);
+    } else {
+      /* to handle a list as the 1st operand of the tuple operator */
+      if (stack_top(0).is_array()) {
+	stack_top(0).array_val()->nested_insert(true);
+      } else if (stack_top(0).is_symbol()) {
+        AttributeValue* av = lookup_symval(&stack_top(0));
+	if (av->is_array()) av->array_val()->nested_insert(true);
+      }
+    }
     _pfoff++;
     if (stack_top().type() == ComValue::CommandType && 
 	!_pfcomvals[_pfoff-1].pedepth()) break;
@@ -311,7 +481,17 @@ int ComTerp::post_eval_expr(int tokcnt, int offtop, int pedepth) {
 	      push_stack(argoffval);
 	    }
 	  }
-	  push_stack(_pfcomvals[offset]);
+	  if (!_pfcomvals[offset].is_blank()) {
+	    push_stack(_pfcomvals[offset]);
+	  } else {
+	    /* to handle a list as the 1st operand of the tuple operator */
+	    if (stack_top(0).is_array()) {
+	      stack_top(0).array_val()->nested_insert(true);
+	    } else if (stack_top(0).is_symbol()) {
+	      AttributeValue* av = lookup_symval(&stack_top(0));
+	      if (av->is_array()) av->array_val()->nested_insert(true);
+	    }
+	  }
 	}
 	tokcnt--;
 	offset++;
@@ -320,6 +500,7 @@ int ComTerp::post_eval_expr(int tokcnt, int offtop, int pedepth) {
 	if (stack_top().is_type(ComValue::CommandType) 
 	    && stack_top().pedepth() == pedepth) break;
       }
+      // if (!stack_top().is_symbol())
       eval_expr_internals(pedepth);
       
     }
@@ -327,34 +508,42 @@ int ComTerp::post_eval_expr(int tokcnt, int offtop, int pedepth) {
   return FUNCOK;
 }
 
-boolean ComTerp::skip_func(ComValue* topval, int& offset) {
+boolean ComTerp::skip_func(ComValue* topval, int& offset, int offlimit) {
   ComValue* sv = topval + offset;
   int nargs = sv->narg();
   int nkeys = sv->nkey();
+  if (offlimit == offset) {
+    cerr << "offlimit hit by ComTerp::skip_func\n";
+    return false;
+  }
   offset--;
   while(nargs>0 || nkeys>0) {
     ComValue* nv = topval + offset;
     int tokcnt;
     if (nv->is_type(ComValue::KeywordType)) {
-      skip_key(topval, offset, tokcnt);
+      skip_key(topval, offset, offlimit, tokcnt);
       nkeys--;
       nargs -= tokcnt ? 1 : 0;
     } else {
-      skip_arg(topval, offset, tokcnt);
+      skip_arg(topval, offset, offlimit, tokcnt);
       nargs--;
     }
   }
   return true;
 }
 
-boolean ComTerp::skip_key(ComValue* topval, int& offset, int& tokcnt) {
+boolean ComTerp::skip_key(ComValue* topval, int& offset, int offlimit, int& tokcnt) {
   ComValue& curr = *(topval+offset);
   tokcnt = 0;
   if (curr.is_type(ComValue::KeywordType)) {
+    if (offlimit == offset) {
+      cerr << "offlimit hit by ComTerp::skip_key\n";
+      return false;
+    }
     offset--;
     if (curr.keynarg_val()) {
       int subtokcnt;
-      skip_arg(topval, offset, subtokcnt);
+      skip_arg(topval, offset, offlimit, subtokcnt);
       tokcnt += subtokcnt;
     }
 
@@ -363,7 +552,7 @@ boolean ComTerp::skip_key(ComValue* topval, int& offset, int& tokcnt) {
   return false;
 }
 
-boolean ComTerp::skip_arg(ComValue* topval, int& offset, int& tokcnt) {
+boolean ComTerp::skip_arg(ComValue* topval, int& offset, int offlimit, int& tokcnt) {
   tokcnt = 0;
   ComValue& curr = *(topval+offset);
   if (curr.is_type(ComValue::KeywordType)) {
@@ -372,23 +561,49 @@ boolean ComTerp::skip_arg(ComValue* topval, int& offset, int& tokcnt) {
   } else if (curr.is_type(ComValue::UnknownType)) {
     cerr << "unexpected unknown found by ComTerp::skip_arg\n";
     return false;
+  } else if (curr.is_type(ComValue::BlankType)) {
+    if (offlimit == offset) {
+      cerr << "offlimit hit by ComTerp::skip_arg\n";
+      return false;
+    }
+    offset--;
+    boolean val = skip_arg(topval, offset, offlimit, tokcnt);
+    tokcnt++;
+    return val;
   } else {
+    if (offlimit == offset) {
+      cerr << "offlimit hit by ComTerp::skip_arg\n";
+      return false;
+    }
     offset--;
     tokcnt++;
+
     if (curr.narg() || curr.nkey()) {
       int count = 0;
       while (count<(curr.narg() + curr.nkey())) {
 	ComValue& next = *(topval+offset);
 	int subtokcnt = 0;
 	if (next.is_type(ComValue::KeywordType)) {
-	  skip_key(topval, offset, subtokcnt);
+	  skip_key(topval, offset, offlimit, subtokcnt);
 	  tokcnt += subtokcnt + 1;
 	  if (subtokcnt) count++;
 	} else if (next.is_type(ComValue::CommandType) ||
 		   next.is_type(ComValue::SymbolType)) {
-	  skip_arg(topval, offset, subtokcnt);
+	  skip_arg(topval, offset, offlimit, subtokcnt);
 	  tokcnt += subtokcnt;
+	} else if (next.is_type(ComValue::BlankType)) {
+	  if (offlimit == offset) {
+	    cerr << "offlimit hit by ComTerp::skip_arg\n";
+	    return false;
+	  }
+	  offset--;
+	  skip_arg(topval, offset, offlimit, subtokcnt);
+	  tokcnt += subtokcnt+1;
 	} else {
+	  if (offlimit == offset) {
+	    cerr << "offlimit hit by ComTerp::skip_arg\n";
+	    return false;
+	  }
 	  offset--;
 	  tokcnt++;
 	}
@@ -514,45 +729,94 @@ void ComTerp::incr_stack(int n) {
 }
 
 void ComTerp::decr_stack(int n) {
-    for (int i=0; i<n; i++) {
+    for (int i=0; i<n && _stack_top>=0; i++) {
         ComValue& stacktop = _stack[_stack_top--];
 	stacktop.AttributeValue::~AttributeValue();
     }
 }
 
 ComValue& ComTerp::pop_stack(boolean lookupsym) {
+  if (!stack_empty()) {
     ComValue& stacktop = _stack[_stack_top--];
     if (lookupsym)
       return lookup_symval(stacktop);
     else 
       return stacktop;
+  } else
+    return ComValue::blankval();
 }
 
 ComValue& ComTerp::lookup_symval(ComValue& comval) {
+    if (comval.bquote()) return comval;
+
     if (comval.type() == ComValue::SymbolType) {
         void* vptr = nil;
-	if (localtable()->find(vptr, comval.symbol_val())) {
-	    comval.assignval(*(ComValue*)vptr);
-	    return comval;
-	} else {
-	    if (_alist) {
-     	        int id = comval.symbol_val();
-	        AttributeValue* aval = _alist->find(id);  
-	        if (aval) {
-		    ComValue newval(*aval);
-		    *&comval = newval;
-		}
-		return comval;
-	    } else 
-	        return ComValue::nullval();
-	}
+
+	if (!comval.global_flag() && localtable()->find(vptr, comval.symbol_val()) ) {
+	  comval.assignval(*(ComValue*)vptr);
+	  return comval;
+	} else  if (_alist) {
+	  int id = comval.symbol_val();
+	  AttributeValue* aval = _alist->find(id);  
+	  if (aval) {
+	    ComValue newval(*aval);
+	    *&comval = newval;
+	  }
+	  return comval;
+	} else if (globaltable()->find(vptr, comval.symbol_val())) {
+	  comval.assignval(*(ComValue*)vptr);
+	  return comval;
+	} else
+	  return ComValue::nullval();
+
+    } else if (comval.is_object(Attribute::class_symid())) {
+
+      comval.assignval(*((Attribute*)comval.obj_val())->Value());
+
     }       
     return comval;
 }
 
+AttributeValue* ComTerp::lookup_symval(ComValue* comval) {
+    if (comval->bquote()) return nil;
+
+    if (comval->type() == ComValue::SymbolType) {
+        void* vptr = nil;
+
+	if (!comval->global_flag() && localtable()->find(vptr, comval->symbol_val()) ) {
+	  return (AttributeValue*)vptr;
+	} else  if (_alist) {
+	  int id = comval->symbol_val();
+	  AttributeValue* aval = _alist->find(id);  
+	  if (aval) {
+	    return aval;
+	  }
+	  return nil;
+	} else if (globaltable()->find(vptr, comval->symbol_val())) {
+	  return (AttributeValue*)vptr;
+	} else
+	  return nil;
+
+    } else if (comval->is_object(Attribute::class_symid())) {
+
+      return ((Attribute*)comval->obj_val())->Value();
+
+    }       
+    return nil;
+}
+
+ComValue& ComTerp::lookup_symval(int symid) {
+  void* vptr = nil;
+  if (localtable()->find(vptr, symid)) {
+    ComValue* valptr = (ComValue*)vptr;
+    return *valptr;
+  } else 
+    return ComValue::nullval();
+}
+
 ComValue& ComTerp::stack_top(int n) {
   if (_stack_top+n < 0 || _stack_top+n >= _stack_siz) {
-    return ComValue::unkval();    
+    return ComValue::blankval();    
   }
   else
     return _stack[_stack_top+n];
@@ -566,13 +830,22 @@ ComValue& ComTerp::pop_symbol() {
         return ComValue::nullval();
 }
 
-int ComTerp::add_command(const char* name, ComFunc* func) {
+int ComTerp::add_command(const char* name, ComFunc* func, const char* alias) {
     int symid = symbol_add((char *)name);
+    func->funcid(symid);
     ComValue* comval = new ComValue();
     comval->type(ComValue::CommandType);
     comval->obj_ref() = (void*)func;
     comval->command_symid(symid);
     localtable()->insert(symid, comval);
+    if (alias) {
+      int alias_symid = symbol_add((char *)alias);
+      ComValue* aliasval = new ComValue();
+      aliasval->type(ComValue::CommandType);
+      aliasval->obj_ref() = (void*)func;
+      aliasval->command_symid(alias_symid, true /* alias */);
+      localtable()->insert(symid, aliasval);
+    }
     return symid;
 }
 
@@ -598,47 +871,71 @@ void ComTerp::quitflag(boolean flag) {
     _quitflag = flag;
 }
 
-int ComTerp::run(boolean once) {
-  int status = 0;
+int ComTerp::run(boolean one_expr, boolean nested) {
+  int status = 1;
   _errbuf[0] = '\0';
+  char errbuf_save[BUFSIZ];
+  errbuf_save[0] = '\0';
   
+
+#if __GNUC__<3
   filebuf fbuf;
   if (handler()) {
-    int fd = max(1, handler()->get_handle());
+    int fd = Math::max(1, handler()->get_handle());
     fbuf.attach(fd);
   } else
     fbuf.attach(fileno(stdout));
+#elif __GNUC__==3 && __GNUC_MINOR__<1
+  fileptr_filebuf fbuf(handler() && handler()->wrfptr() 
+	       ? handler()->wrfptr() : stdout, 
+	       ios_base::out);
+#else
+  fileptr_filebuf fbuf(handler()&&handler()->get_handle()>0 
+		       ? handler()->get_handle() : 1, 
+		       ios_base::out, false, static_cast<size_t>(BUFSIZ));
+#endif
   ostream out(&fbuf);
   boolean eolflag = false;
 
   while (!eof() && !quitflag() && !eolflag) {
     
     if (read_expr()) {
-      eval_expr();
+      status = 0;
+      int top_before = _stack_top;
+      eval_expr(nested);
+      if (top_before == _stack_top)
+	status = 2;
       err_str( _errbuf, BUFSIZ, "comterp" );
       if (strlen(_errbuf)==0) {
 	if (quitflag()) {
 	  status = -1;
 	  break;
-	} else {
+	} else if (!func_for_next_expr() && val_for_next_func().is_null()) {
 	  print_stack_top(out);
-	  out << "\n"; out.flush();
+	  out << "\n"; out.flush(); 
 	}
       } else {
 	out << _errbuf << "\n"; out.flush();
+	strcpy(errbuf_save, _errbuf);
 	_errbuf[0] = '\0';
       }
     } else {
       err_str( _errbuf, BUFSIZ, "comterp" );
       if (strlen(_errbuf)>0) {
 	out << _errbuf << "\n"; out.flush();
+	strcpy(errbuf_save, _errbuf);
 	_errbuf[0] = '\0';
-      } else
+      } else {
 	eolflag = true;
+        if (errbuf_save[0]) strcpy(_errbuf, errbuf_save);
+      }
     }
-    _stack_top = -1;
-    if (once) break;
+    if (!nested) 
+      _stack_top = -1;
+    if (one_expr) break;
   }
+  if (status==1 && _pfnum==0) status=2;
+  if (nested && status!=2) _stack_top--;
   return status;
 }
 
@@ -675,6 +972,12 @@ void ComTerp::add_defaults() {
     add_command("decr", new DecrFunc(this));
     add_command("decr_after", new DecrAfterFunc(this));
 
+    add_command("bit_and", new BitAndFunc(this));
+    add_command("bit_xor", new BitXorFunc(this));
+    add_command("bit_or", new BitOrFunc(this));
+    add_command("bit_not", new BitNotFunc(this));
+    add_command("lshift", new LeftShiftFunc(this));
+    add_command("rshift", new RightShiftFunc(this));
     add_command("and", new AndFunc(this));
     add_command("or", new OrFunc(this));
     add_command("negate", new NegFunc(this));
@@ -686,11 +989,20 @@ void ComTerp::add_defaults() {
     add_command("lt_or_eq", new LessThanOrEqualFunc(this));
 
     add_command("stream", new StreamFunc(this));
+    add_command("concat", new ConcatFunc(this));
     add_command("repeat", new RepeatFunc(this));
     add_command("iterate", new IterateFunc(this));
+    add_command("next", new NextFunc(this));
+    add_command("each", new EachFunc(this));
 
+    add_command("dot", new DotFunc(this));
+    add_command("attrname", new DotNameFunc(this));
+    add_command("attrval", new DotValFunc(this));
+
+    add_command("list", new ListFunc(this));
     add_command("at", new ListAtFunc(this));
     add_command("size", new ListSizeFunc(this));
+    add_command("tuple", new TupleFunc(this));
 
     add_command("sum", new SumFunc(this));
     add_command("mean", new MeanFunc(this));
@@ -713,17 +1025,40 @@ void ComTerp::add_defaults() {
     add_command("sin", new SinFunc(this));
     add_command("tan", new TanFunc(this));
     add_command("sqrt", new SqrtFunc(this));
+    add_command("pi", new PiFunc(this));
+    add_command("radtodeg", new RadToDegFunc(this));
+    add_command("degtorad", new DegToRadFunc(this));
+
+    add_command("floor", new FloorFunc(this));
+    add_command("ceil", new CeilFunc(this));
+    add_command("round", new RoundFunc(this));
 
     add_command("xform", new XformFunc(this));
     add_command("invert", new InvertXformFunc(this));
+    add_command("xpose", new XposeFunc(this));
 
     add_command("cond", new CondFunc(this));
     add_command("seq", new SeqFunc(this));
     add_command("run", new RunFunc(this));
 
     add_command("help", new HelpFunc(this));
+    add_command("trace", new ComterpTraceFunc(this));
+    add_command("pause", new ComterpPauseFunc(this));
+    add_command("step", new ComterpStepFunc(this));
+    add_command("stackheight", new ComterpStackHeightFunc(this));
     add_command("symid", new SymIdFunc(this));
     add_command("symval", new SymValFunc(this));
+    add_command("symbol", new SymbolFunc(this));
+    add_command("symadd", new SymAddFunc(this));
+    add_command("global", new GlobalSymbolFunc(this));
+    add_command("split", new SplitStrFunc(this));
+    add_command("join", new JoinStrFunc(this));
+
+    add_command("type", new TypeSymbolFunc(this));
+    add_command("class", new ClassSymbolFunc(this));
+
+    add_command("bquote", new BackQuoteFunc(this));
+
     add_command("postfix", new PostFixFunc(this));
     add_command("posteval", new PostEvalFunc(this));
 
@@ -760,21 +1095,30 @@ int ComTerp::runfile(const char* filename) {
     int tokoff = _pfoff;
 
     /* swap in input pointer and function */
+#if 0
     void* save_inptr = _inptr;
     infuncptr save_infunc = _infunc;
     outfuncptr save_outfunc = _outfunc;
+#else
+    push_servstate();
+#endif
     FILE* fptr = fopen(filename, "r");
     _inptr = fptr;
     _outfunc = nil;
+    if (!fptr) cerr << "unable to run from file " << filename << "\n";
     
 
     ComValue* retval = nil;
     int status = 0;
-    while( !feof(fptr)) {
+    while( fptr && !feof(fptr)) {
 	if (read_expr()) {
 	    if (eval_expr(true)) {
 	        err_print( stderr, "comterp" );
+#if __GNUC__<3
 	        filebuf obuf(1);
+#else
+	        fileptr_filebuf obuf(stdout, ios_base::out);
+#endif
 		ostream ostr(&obuf);
 		ostr << "err\n";
 		ostr.flush();
@@ -789,9 +1133,13 @@ int ComTerp::runfile(const char* filename) {
 	}
     }
 
+#if 0
     _inptr = save_inptr;
     _infunc = save_infunc;
     _outfunc = save_outfunc;
+#else
+    pop_servstate();
+#endif
 
     load_postfix(tokbuf, toklen, tokoff);
     delete tokbuf;
@@ -844,7 +1192,11 @@ void ComTerp::list_commands(ostream& out, boolean sorted) {
 	rowcnt = 0;
 	out << "\n";
       } else
+#if 0   
 	out << "\t";
+#else
+      for(int t=0; t<=tlen; t++) out << ' ';
+#endif
     }
     delete funcids;
   }
@@ -880,13 +1232,14 @@ int* ComTerp::get_commands(int& ncomm, boolean sort) {
   if (sort) {
     int* sortedbuffer = new int[ncomm];
     int i = 0;  /* operators first */
-    for (int j=0; j< ncomm; j++) sortedbuffer[j] = -1;
-    for (int j=0; j< ncomm; j++)
+    int j;
+    for (j=0; j< ncomm; j++) sortedbuffer[j] = -1;
+    for (j=0; j< ncomm; j++)
       if (!isalpha(*symbol_pntr(buffer[j])))
 	  sortedbuffer[i++] = buffer[j];
     if (i != opercnt) cerr << "bad number of operators\n";
       
-    for (int j=0; j<ncomm; j++) {
+    for (j=0; j<ncomm; j++) {
       if (!isalpha(*symbol_pntr(buffer[j]))) continue;
 
       /* count the number of strings greater than this one */
@@ -901,7 +1254,7 @@ int* ComTerp::get_commands(int& ncomm, boolean sort) {
 
     /* one more pass over the sorted buffer to remove duplicates */
     int copydist = 0;
-    for (int j=0; j<ncomm; j++) {
+    for (j=0; j<ncomm; j++) {
       if (sortedbuffer[j]<0) 
 	copydist++;
       else 
@@ -959,3 +1312,106 @@ void ComTerp::push_funcstate(ComFuncState& funcstate) {
   ComFuncState* sfs = _fsstack + _fsstack_top;
   *sfs = ComFuncState(funcstate);
 }
+
+void ComTerp::func_for_next_expr(ComFunc* func) {
+  if (!_func_for_next_expr)
+    _func_for_next_expr = func;
+}
+
+ComFunc* ComTerp::func_for_next_expr() {
+  return _func_for_next_expr;
+}
+
+void ComTerp::val_for_next_func(ComValue& val) {
+  if (_val_for_next_func) {
+    delete _val_for_next_func;
+  }
+  _val_for_next_func = new ComValue(val);
+}
+
+ComValue& ComTerp::val_for_next_func() {
+  if (_val_for_next_func) {
+    return *_val_for_next_func;
+  } else
+    return ComValue::nullval();
+}
+
+void ComTerp::clr_val_for_next_func() {
+  delete _val_for_next_func;
+  _val_for_next_func = nil;
+}
+
+ComTerpState* ComTerp::top_servstate() {
+  return _ctsstack_top < 0 ? nil : _ctsstack+_ctsstack_top;
+}
+
+void ComTerp::pop_servstate() {
+  if (_ctsstack_top >=0) {
+
+    ComTerpState* cts_state = top_servstate();
+
+    /* clean up */
+    delete _buffer;
+    delete _pfbuf;
+    delete [] _pfcomvals;
+
+    /* restore copies of everything */
+    _pfbuf = cts_state->pfbuf();
+    _pfnum = cts_state->pfnum();
+    _pfoff = cts_state->pfoff();
+    _bufptr = cts_state->bufptr();
+    _linenum = cts_state->linenum();
+    //    _just_reset = cts_state->just_reset();
+    _buffer = cts_state->buffer();
+    _pfcomvals = cts_state->pfcomvals();
+    _infunc = cts_state->infunc();
+    _eoffunc = cts_state->eoffunc();
+    _errfunc = cts_state->errfunc();
+    _inptr = cts_state->inptr();
+    
+    _ctsstack_top--;
+  }
+}
+
+void ComTerp::push_servstate() {
+  ComTerpState cts_state;
+
+  /* save copies of everything */
+  cts_state.pfbuf() = _pfbuf;
+  cts_state.pfnum() = _pfnum;
+  cts_state.pfoff() = _pfoff;
+  cts_state.bufptr() = _bufptr;
+  cts_state.linenum() = _linenum;
+  //  cts_state.just_reset() = _just_reset;
+  cts_state.buffer() = _buffer;
+  cts_state.pfcomvals() = _pfcomvals;
+  cts_state.infunc() = _infunc;
+  cts_state.eoffunc() = _eoffunc;
+  cts_state.errfunc() = _errfunc;
+  cts_state.inptr() = _inptr;
+
+  /* re-initialize */
+  if(dmm_calloc((void**)&_pfbuf, _pfsiz, sizeof(postfix_token)) != 0) 
+    KANRET("error in call to dmm_calloc");
+  _pfnum = _pfoff = 0;
+  _buffer = new char[_bufsiz];
+  _bufptr = 0;
+  _linenum = 0;
+  // _just_reset = false;
+  _pfcomvals = nil;
+
+  if (_ctsstack_top+1 == _ctsstack_siz) {
+    _ctsstack_siz *= 2;
+    dmm_realloc_size(sizeof(ComTerpState));
+    if(dmm_realloc((void**)&_ctsstack, (unsigned long)_ctsstack_siz) != 0) {
+      KANRET("error in call to dmm_realloc");
+      return;
+    }
+  } 
+  _ctsstack_top++;
+  ComTerpState* ctss = _ctsstack + _ctsstack_top;
+  *ctss = cts_state;
+}
+
+
+
