@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1994,1995,1999 Vectaport Inc.
+ * Copyright (c) 2001 Scott E. Johnston
+ * Copyright (c) 1994,1995,1999,2000 Vectaport Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and
  * its documentation for any purpose is hereby granted without fee, provided
@@ -24,6 +25,11 @@
 #include <ComTerp/numfunc.h>
 #include <ComTerp/comvalue.h>
 #include <ComTerp/comterp.h>
+#include <Unidraw/iterator.h>
+#include <Attribute/attrlist.h>
+#include <OS/math.h>
+#include <math.h>
+#include <string.h>
 
 #define TITLE "NumFunc"
 
@@ -35,6 +41,14 @@ NumFunc::NumFunc(ComTerp* comterp) : ComFunc(comterp) {
 void NumFunc::promote(ComValue& op1, ComValue& op2) {
     if (op1.type() == op2.type()) return;
 
+#if 0
+    if (op1.is_unknown() || op2.is_unknown()) {
+      op1.type(ComValue::UnknownType);
+      op2.type(ComValue::UnknownType);
+      return;
+    }
+#endif
+
     boolean op1bigger = op1.type() > op2.type();
     ComValue* greater = op1bigger ? &op1 : &op2;
     ComValue* lesser =  op1bigger ? &op2 : &op1;
@@ -171,6 +185,12 @@ void AddFunc::execute() {
     promote(operand1, operand2);
     ComValue result(operand1);
 
+    if (operand1.is_unknown() || operand2.is_unknown()) {
+      reset_stack();
+      push_stack(ComValue::nullval());
+      return;
+    }
+    
     switch (result.type()) {
     case ComValue::CharType:
 	result.char_ref() = operand1.char_val() + operand2.char_val();
@@ -212,11 +232,42 @@ void AddFunc::execute() {
 	  result.string_ref()  = symbol_add(buffer);
 	}
 	break;
+    case ComValue::ArrayType: 
+        {
+	  if (operand2.is_array()) {
+	    Resource::unref(result.array_val());
+	    result.array_ref() = 
+	      AddFunc::matrix_add(operand1.array_val(), operand2.array_val());
+	    Resource::ref(result.array_val());
+	  }
+	  else 
+	    result.type(ComValue::UnknownType); // nil
+        }
+        break;
+
     }
     reset_stack();
     push_stack(result);
 }
 
+
+AttributeValueList* AddFunc::matrix_add(AttributeValueList* list1,
+					AttributeValueList* list2) {
+  AttributeValueList* sum = new AttributeValueList();
+  Iterator it1, it2;
+  list1->First(it1);
+  list2->First(it2);
+  while (!list1->Done(it1) && !list2->Done(it2)) {
+    push_stack(*list1->GetAttrVal(it1));
+    push_stack(*list2->GetAttrVal(it2));
+    exec(2, 0);
+    sum->Append(new AttributeValue(comterp()->pop_stack()));
+    list1->Next(it1);
+    list2->Next(it2);
+  }
+  return sum;
+}
+
 SubFunc::SubFunc(ComTerp* comterp) : NumFunc(comterp) {
 }
 
@@ -226,6 +277,12 @@ void SubFunc::execute() {
     promote(operand1, operand2);
     ComValue result(operand1);
 
+    if (operand1.is_unknown() || operand2.is_unknown()) {
+      reset_stack();
+      push_stack(ComValue::nullval());
+      return;
+    }
+    
     switch (result.type()) {
     case ComValue::CharType:
 	result.char_ref() = operand1.char_val() - operand2.char_val();
@@ -268,6 +325,13 @@ MinusFunc::MinusFunc(ComTerp* comterp) : NumFunc(comterp) {
 void MinusFunc::execute() {
     ComValue& operand1 = stack_arg(0);
     ComValue result(operand1);
+
+    if (operand1.is_unknown()) {
+      reset_stack();
+      push_stack(ComValue::nullval());
+      return;
+    }
+    
     switch (result.type()) {
     case ComValue::CharType:
 	result.char_ref() = - operand1.char_val();
@@ -313,6 +377,12 @@ void MpyFunc::execute() {
     promote(operand1, operand2);
     ComValue result(operand1);
 
+    if (operand1.is_unknown() || operand2.is_unknown()) {
+      reset_stack();
+      push_stack(ComValue::nullval());
+      return;
+    }
+    
     switch (result.type()) {
     case ComValue::CharType:
 	result.char_ref() = operand1.char_val() * operand2.char_val();
@@ -344,11 +414,163 @@ void MpyFunc::execute() {
     case ComValue::DoubleType:
 	result.double_ref() = operand1.double_val() * operand2.double_val();
 	break;
+    case ComValue::ArrayType: 
+        {
+	  if (operand2.is_array()) {
+	    Resource::unref(result.array_val());
+	    AttributeValueList* avl = 
+	      MpyFunc::matrix_mpy(operand1.array_val(), operand2.array_val());
+	    if (avl) {
+	      result.array_ref() = avl;
+	      Resource::ref(result.array_val());
+	    } else
+	      result.type(ComValue::UnknownType); // nil
+	  }
+	  else 
+	    result.type(ComValue::UnknownType); // nil
+        }
+        break;
     }
+
     reset_stack();
     push_stack(result);
 }
 
+AttributeValueList* MpyFunc::matrix_mpy(AttributeValueList* list1,
+					AttributeValueList* list2) {
+
+  static AddFunc addfunc(comterp());
+  Iterator it1, it2;
+  list1->First(it1);
+  list2->First(it2);
+  
+  // extract dimensions
+  int i1max, j1max;
+  int i2max, j2max;
+  i1max = list1->Number();
+  i2max = list2->Number();
+  j1max = list1->GetAttrVal(it1)->is_array() &&
+    list1->GetAttrVal(it1)->array_val() ? 
+    list1->GetAttrVal(it1)->array_val()->Number() : 0;
+  j2max = list2->GetAttrVal(it2)->is_array() &&
+    list1->GetAttrVal(it2)->array_val() ? 
+    list1->GetAttrVal(it2)->array_val()->Number() : 0;
+
+  /* ensure inner dimension is the same */
+  /* allow for vector argument on rhs */
+  if (j1max != i2max && (j1max || i2max!=1)) return nil;
+
+  /* ensure each row is of equal length */
+  list1->First(it1);
+  list1->Next(it1);
+  while (!list1->Done(it1)) {
+    int jlen = list1->GetAttrVal(it1)->is_array() &&
+      list1->GetAttrVal(it1)->array_val() ? 
+      list1->GetAttrVal(it1)->array_val()->Number() : 0;
+    if (jlen != j1max) return nil;
+    list1->Next(it1);
+  }
+  list2->First(it2);
+  list2->Next(it2);
+  while (!list2->Done(it2)) {
+    int jlen = list2->GetAttrVal(it2)->is_array() &&
+      list2->GetAttrVal(it2)->array_val() ? 
+      list2->GetAttrVal(it2)->array_val()->Number() : 0;
+    if (jlen != j2max) return nil;
+    list2->Next(it2);
+  }
+  
+
+  AttributeValueList* product = new AttributeValueList();
+
+  int i3max = i1max;
+  int j3max = j2max;
+
+  /* loop over output rows */
+  Iterator iti1, itj1;
+  list1->First(iti1);
+
+  for (int i3=0; i3<i3max; i3++) {
+    AttributeValueList* prodrow = new AttributeValueList();
+    product->Append(new AttributeValue(prodrow));
+    AttributeValue* row1v = list1->GetAttrVal(iti1);
+    AttributeValueList* row1 = row1v && row1v->is_array() ? 
+      row1v->array_val() : nil;
+
+    if (j3max) {
+      /* loop over output columns */
+      for (int j3=0; j3<j3max; j3++) {
+	
+	if (row1) row1->First(itj1);
+	
+	/* generate inner product */
+	for (int n=0; n<Math::max(j1max,1); n++) {
+	  
+	  /* locate the value from the second matrix */
+	  Iterator iti2, itj2;
+	  list2->First(iti2);
+	  for (int i=0; i<n; i++) list2->Next(iti2);
+	  AttributeValue* row2v = list1->GetAttrVal(iti2);
+	  AttributeValueList* row2 = row2v && row2v->is_array() ? 
+	    row2v->array_val() : nil;
+	  if (row2) {
+	    row2->First(itj2);
+	    for (int j=0; j<j3; j++) row2->Next(itj2);
+	  }
+	  if ((row1 || !j1max) && row2) {
+	    if (row1) 
+	      comterp()->push_stack(*row1->GetAttrVal(itj1));
+	    else
+	      comterp()->push_stack(*row1v);
+	    comterp()->push_stack(*row2->GetAttrVal(itj2));
+	    exec(2,0);
+	    if (n) addfunc.exec(2,0);
+	  }
+	  
+	  if (row1) row1->Next(itj1);
+	}
+	
+	prodrow->Append(new AttributeValue(comterp()->pop_stack()));
+      }
+      /* done looping over output columsn */
+      
+    } else {
+      /* handle single output column */
+
+      if (row1) row1->First(itj1);
+	
+      /* generate inner product */
+      for (int n=0; n<Math::max(j1max,1); n++) {
+	  
+	/* locate the value from the lhs vector */
+	Iterator iti2;
+	list2->First(iti2);
+	for (int i=0; i<n; i++) list2->Next(iti2);
+	AttributeValue* val2v = list1->GetAttrVal(iti2);
+	if ((row1 || !j1max) && val2v) {
+	  if (row1) 
+	    comterp()->push_stack(*row1->GetAttrVal(itj1));
+	  else
+	    comterp()->push_stack(*row1v);
+	  comterp()->push_stack(*val2v);
+	  exec(2,0);
+	  if (n) addfunc.exec(2,0);
+	}
+	
+	if (row1) row1->Next(itj1);
+      }
+      
+      prodrow->Append(new AttributeValue(comterp()->pop_stack()));
+    }
+
+    list1->Next(iti1);
+  }
+  /* done looping over output rows */
+
+
+  return product;
+}
+
 DivFunc::DivFunc(ComTerp* comterp) : NumFunc(comterp) {
 }
 
@@ -356,6 +578,13 @@ void DivFunc::execute() {
     ComValue& operand1 = stack_arg(0);
     ComValue& operand2 = stack_arg(1);
     promote(operand1, operand2);
+
+    if (operand1.is_unknown() || operand2.is_unknown()) {
+      reset_stack();
+      push_stack(ComValue::nullval());
+      return;
+    }
+    
     ComValue result(operand1);
 
     switch (result.type()) {
@@ -433,6 +662,12 @@ void ModFunc::execute() {
     promote(operand1, operand2);
     ComValue result(operand1);
 
+    if (operand1.is_unknown() || operand2.is_unknown()) {
+      reset_stack();
+      push_stack(ComValue::nullval());
+      return;
+    }
+    
     switch (result.type()) {
     case ComValue::CharType:
 	if (operand2.char_val()!=0)
@@ -508,6 +743,12 @@ void MinFunc::execute() {
     promote(operand1, operand2);
     ComValue result(operand1);
 
+    if (operand1.is_unknown() || operand2.is_unknown()) {
+      reset_stack();
+      push_stack(ComValue::nullval());
+      return;
+    }
+    
     switch (result.type()) {
     case ComValue::CharType:
 	result.char_ref() =  operand1.char_val() < operand2.char_val() 
@@ -549,6 +790,9 @@ void MinFunc::execute() {
 	result.double_ref() =  operand1.double_val() < operand2.double_val() 
 	  ? operand1.double_val() : operand2.double_val();
 	break;
+    case ComValue::UnknownType:
+	result.assignval(operand2);
+	break;
     }
     reset_stack();
     push_stack(result);
@@ -563,6 +807,12 @@ void MaxFunc::execute() {
     promote(operand1, operand2);
     ComValue result(operand1);
 
+    if (operand1.is_unknown() || operand2.is_unknown()) {
+      reset_stack();
+      push_stack(ComValue::nullval());
+      return;
+    }
+    
     switch (result.type()) {
     case ComValue::CharType:
 	result.char_ref() =  operand1.char_val() > operand2.char_val() 
@@ -604,6 +854,10 @@ void MaxFunc::execute() {
 	result.double_ref() =  operand1.double_val() > operand2.double_val() 
 	  ? operand1.double_val() : operand2.double_val();
 	break;
+    case ComValue::UnknownType:
+	result.assignval(operand2);
+	break;
+     
     }
     reset_stack();
     push_stack(result);
@@ -616,6 +870,12 @@ void AbsFunc::execute() {
     ComValue& operand1 = stack_arg(0);
     ComValue result(operand1);
 
+    if (operand1.is_unknown()) {
+      reset_stack();
+      push_stack(ComValue::nullval());
+      return;
+    }
+    
     switch (result.type()) {
     case ComValue::CharType:
 	result.char_ref() =  operand1.char_val() < 0 
@@ -651,7 +911,8 @@ CharFunc::CharFunc(ComTerp* comterp) : ComFunc(comterp) {}
 
 void CharFunc::execute() {
     ComValue& operand = stack_arg(0);
-    ComValue result(operand.char_val(), ComValue::CharType);
+    ComValue result(operand.char_val(), 
+		    operand.is_nil() ? ComValue::UnknownType : ComValue::CharType);
     reset_stack();
     push_stack(result);
 }
@@ -660,7 +921,8 @@ ShortFunc::ShortFunc(ComTerp* comterp) : ComFunc(comterp) {}
 
 void ShortFunc::execute() {
     ComValue& operand = stack_arg(0);
-    ComValue result(operand.short_val(), ComValue::ShortType);
+    ComValue result(operand.short_val(), 
+		    operand.is_nil() ? ComValue::UnknownType : ComValue::ShortType);
     reset_stack();
     push_stack(result);
 }
@@ -669,7 +931,8 @@ IntFunc::IntFunc(ComTerp* comterp) : ComFunc(comterp) {}
 
 void IntFunc::execute() {
     ComValue& operand = stack_arg(0);
-    ComValue result(operand.int_val(), ComValue::IntType);
+    ComValue result(operand.int_val(),  
+		    operand.is_nil() ? ComValue::UnknownType : ComValue::IntType);
     reset_stack();
     push_stack(result);
 }
@@ -679,6 +942,7 @@ LongFunc::LongFunc(ComTerp* comterp) : ComFunc(comterp) {}
 void LongFunc::execute() {
     ComValue& operand = stack_arg(0);
     ComValue result(operand.long_val());
+    if (operand.is_nil()) result.type(ComValue::UnknownType);
     reset_stack();
     push_stack(result);
 }
@@ -688,6 +952,7 @@ FloatFunc::FloatFunc(ComTerp* comterp) : ComFunc(comterp) {}
 void FloatFunc::execute() {
     ComValue& operand = stack_arg(0);
     ComValue result(operand.float_val());
+    if (operand.is_nil()) result.type(ComValue::UnknownType);
     reset_stack();
     push_stack(result);
 }
@@ -697,7 +962,108 @@ DoubleFunc::DoubleFunc(ComTerp* comterp) : ComFunc(comterp) {}
 void DoubleFunc::execute() {
     ComValue& operand = stack_arg(0);
     ComValue result(operand.double_val());
+    if (operand.is_nil()) result.type(ComValue::UnknownType);
     reset_stack();
     push_stack(result);
 }
 
+FloorFunc::FloorFunc(ComTerp* comterp) : NumFunc(comterp) {
+}
+
+void FloorFunc::execute() {
+    ComValue& operand1 = stack_arg(0);
+    ComValue result(operand1);
+    switch (result.type()) {
+    case ComValue::CharType:
+    case ComValue::UCharType:
+    case ComValue::ShortType:
+    case ComValue::UShortType:
+    case ComValue::IntType:
+    case ComValue::UIntType:
+    case ComValue::LongType:
+    case ComValue::ULongType:
+      break;
+    case ComValue::FloatType:
+      {
+	ComValue val((long)floor((double) operand1.float_val()));
+	result.assignval(val);
+      }
+      break;
+    case ComValue::DoubleType:
+      {
+        ComValue val((long)floor(operand1.double_val()));
+	result.assignval(val);
+      }
+      break;
+    }
+    reset_stack();
+    push_stack(result);
+}
+
+CeilFunc::CeilFunc(ComTerp* comterp) : NumFunc(comterp) {
+}
+
+void CeilFunc::execute() {
+    ComValue& operand1 = stack_arg(0);
+    ComValue result(operand1);
+    switch (result.type()) {
+    case ComValue::CharType:
+    case ComValue::UCharType:
+    case ComValue::ShortType:
+    case ComValue::UShortType:
+    case ComValue::IntType:
+    case ComValue::UIntType:
+    case ComValue::LongType:
+    case ComValue::ULongType:
+      break;
+    case ComValue::FloatType:
+      {
+	ComValue val((long)ceil((double) operand1.float_val()));
+	result.assignval(val);
+      }
+      break;
+    case ComValue::DoubleType:
+      {
+        ComValue val((long)ceil(operand1.double_val()));
+	result.assignval(val);
+      }
+      break;
+    }
+    reset_stack();
+    push_stack(result);
+}
+
+RoundFunc::RoundFunc(ComTerp* comterp) : NumFunc(comterp) {
+}
+
+void RoundFunc::execute() {
+    ComValue& operand1 = stack_arg(0);
+    ComValue result(operand1);
+    switch (result.type()) {
+    case ComValue::CharType:
+    case ComValue::UCharType:
+    case ComValue::ShortType:
+    case ComValue::UShortType:
+    case ComValue::IntType:
+    case ComValue::UIntType:
+    case ComValue::LongType:
+    case ComValue::ULongType:
+      break;
+    case ComValue::FloatType:
+      {
+	ComValue val((long)floor((double) operand1.float_val()+0.5));
+	result.assignval(val);
+      }
+      break;
+    case ComValue::DoubleType:
+      {
+        ComValue val((long)floor(operand1.double_val()+0.5));
+	result.assignval(val);
+      }
+      break;
+    }
+    reset_stack();
+    push_stack(result);
+}
+
+
