@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2000 IET Inc.
  * Copyright (c) 1993-1995 Vectaport Inc.
  * Copyright (c) 1989 Triple Vision, Inc.
  *
@@ -74,7 +75,8 @@ Summary:
 */
 
 int lexscan(void * infile,char * (*infunc)(),int (*eoffunc)(), int (*errfunc)(), 
-	    void * outfile,int (*outfunc)(),char * begcmt,char * endcmt,
+	    void * outfile,int (*outfunc)(),
+	    char * begcmt,char * endcmt, char linecmt,
             char * buffer,unsigned bufsiz,unsigned * bufptr,char * token,
 	    unsigned toksiz, unsigned * toklen,unsigned * toktype,
 	    unsigned * tokstart,unsigned * linenum )
@@ -103,6 +105,7 @@ int           (*outfunc)();/* I   Function for writing output text
 				  (typically `fputs`). */
 char *	        begcmt    ;/* I   String that begins comment. */
 char *          endcmt    ;/* I   String that ends comment. */
+char            linecmt   ;/* I   Character to start comment line.  */
 char *          buffer    ;/* I   Buffer or file I/O.          */
 unsigned        bufsiz    ;/* I   Size of `buffer` in bytes.   */
 unsigned *      bufptr    ;/* O   Current location in `buffer`.*/
@@ -149,6 +152,8 @@ unsigned double_state = FLOAT_INTEGER;
 BOOLEAN long_num = FALSE;       /* Indicates long integer to be used */
 unsigned token_state = TOK_WHITESPACE;
 				/* Internal token state variable */
+static unsigned token_state_save = TOK_WHITESPACE;
+				/* variable to save token state between calls */
 unsigned begcmt_len =           /* Number of characters in comment beginning */
    (begcmt != NULL ? strlen(begcmt) : 0 );
 unsigned endcmt_len =           /* Number of characters in comment ending */
@@ -156,6 +161,7 @@ unsigned endcmt_len =           /* Number of characters in comment ending */
 unsigned no_comment =           /* TRUE if comments are disabled */
    (begcmt_len == 0 || endcmt_len == 0);
 int index;
+char* infunc_retval;
 
 
 /* ----------------------------------------------------------------------- */
@@ -172,6 +178,8 @@ int index;
    *toktype = TOK_NONE;
    *toklen = 0;
    *tokstart = 0;
+   token_state = token_state_save;
+   token_state_save = TOK_WHITESPACE;
 
 /* Initialize if linenumber is 0 */
    if( *linenum == 0 ) {
@@ -236,7 +244,12 @@ int index;
 	     (*outfunc) ( "> ", outfile);
 	   _continuation_prompt = 0;
 	 }
-	 if( (*infunc)( buffer, bufsiz, infile ) == NULL ) {
+	 if (linecmt)
+	   while( (infunc_retval = (*infunc)( buffer, bufsiz, infile )) != NULL && 
+		  buffer[0] == linecmt) {} /* skip all script comments */
+	 else
+	   infunc_retval = (*infunc)( buffer, bufsiz, infile );
+	 if( infunc_retval == NULL ) {
 	    if( *toklen > 0 )
 	       goto token_return;
 	    else {
@@ -291,6 +304,12 @@ int index;
       /* Reset pointer to front of buffer */
 	 *bufptr = 0;
          CURR_CHAR = buffer[0];
+	 if (CURR_CHAR == '\0') {
+	   if (token_state == TOK_COMMENT)
+	     token_state_save = token_state;
+	    *linenum--;
+	    return  FUNCOK;
+	 }
 
       /* Echo source line if so desired */
 #if 0
