@@ -60,10 +60,13 @@
 #include <InterViews/raster.h>
 #include <InterViews/transformer.h>
 
+#include <OS/math.h>
+
 #include <ctype.h>
 #include <stdio.h>
 #include <stream.h>
 #include <string.h>
+#include <fstream.h>
 
 
 /*****************************************************************************/
@@ -90,9 +93,9 @@ static unsigned int hexintmap[] = {
 static const char* HexEncode (
     ColorIntensity ir, ColorIntensity ig, ColorIntensity ib
 ) {
-    unsigned int r = round(ir * color_base);
-    unsigned int g = round(ig * color_base);
-    unsigned int b = round(ib * color_base);
+    unsigned int r = Math::round(ir * color_base);
+    unsigned int g = Math::round(ig * color_base);
+    unsigned int b = Math::round(ib * color_base);
 
     static char enc[hex_encode+1];
     enc[hex_encode] = '\0';
@@ -178,6 +181,7 @@ boolean OverlayCatalog::Retrieve (const char* filename, Component*& comp) {
         _valid = true;
 
     } else {
+#if __GNUC__<3	  
         filebuf fbuf;
 	if (strcmp(name, "-") == 0) {
 	    _valid = fbuf.attach(fileno(stdin)) != 0;
@@ -192,6 +196,24 @@ boolean OverlayCatalog::Retrieve (const char* filename, Component*& comp) {
 		else if (strcmp(name+namelen-2,".Z")==0) name[namelen-2] = '\0';
 	    }
 	}
+#else
+	boolean stdin_flag = strcmp(name, "-")==0;
+	if (!stdin_flag) {
+	  fptr = fopen(name, "r");
+	  fptr = fptr ? OvImportCmd::CheckCompression(fptr, name, compressed) : nil;
+	  _valid = fptr != nil;
+	  if (compressed) {
+	    int namelen = strlen(name);
+	    if (strcmp(name+namelen-3,".gz")==0) name[namelen-3] = '\0';
+	    else if (strcmp(name+namelen-2,".Z")==0) name[namelen-2] = '\0';
+	  }
+	} else {
+	  _valid = 1;
+	  name = nil;
+	}
+	if (!_valid && !ParamList::urltest(name)) return false;
+        fileptr_filebuf fbuf(stdin_flag ? stdin : fptr, ios_base::in);
+#endif
 	
         if (_valid || ParamList::urltest(name)) {
 	    istream in(&fbuf);
@@ -317,6 +339,10 @@ void OverlayCatalog::PSReadChildren (istream& in, GraphicComp* comp) {
 	else if (strcmp(_buf, "SSten") == 0)    child = ReadSStencil(in);
 	else if (strcmp(_buf, "FSten") == 0)    child = ReadFStencil(in);
 	else if (strcmp(_buf, "Rast") == 0)     child = ReadRaster(in);
+	else if (strcmp(_buf, "ColorRast") ==0) {
+	  child = nil; 
+	  cerr << "Support for reading idraw PostScript with color-printer ready rasters not yet available.\n"; 
+	}
 	else if (strcmp(_buf, "eop") == 0)      break;
 
 	else {
