@@ -58,11 +58,14 @@
 #include <string.h>
 #include <sys/param.h>
 #include <unistd.h>
+#include <stdlib.h>
 
+#if __GNUC__<2 || __GNUC__==2 && __GNUC_MINOR__<96
 extern "C" 
 {
   char* realpath();
 }
+#endif
 
 /*****************************************************************************/
 
@@ -82,6 +85,8 @@ static void NullGS (Graphic* g) { FullGraphic null; *g = null; }
 
 /*****************************************************************************/
 
+int OverlayComp::_symid = -1;
+
 ParamList* OverlayComp::_overlay_comp_params = nil;
 
 OverlayComp::OverlayComp (Graphic* g, OverlayComp* parent) : GraphicComp(g)
@@ -120,9 +125,10 @@ AttributeList* OverlayComp::GetAttributeList() {
 #endif
 
 void OverlayComp::SetAttributeList(AttributeList* al) {
+  if (_attrlist) 
     Unref(_attrlist);
-    _attrlist = al;
-    Resource::ref(_attrlist);
+  _attrlist = al;
+  Resource::ref(_attrlist);
 
 #if 0 // experimentation with attribute changing
     if (al) {
@@ -192,6 +198,7 @@ void OverlayComp::GrowParamList(ParamList* pl) {
 }
 
 OverlayView* OverlayComp::FindView (Viewer* viewer) {
+    if (!_views) return nil;
     for (UList* u = _views->First(); u != _views->End(); u = u->Next()) {
 	ComponentView* compview = View(u);
 	if (compview->IsA(OVERLAY_VIEW) && 
@@ -297,6 +304,13 @@ boolean OverlayComp::GetByPathnameFlag() {
 void OverlayComp::SetByPathnameFlag(boolean flag) {
 }
 
+boolean OverlayComp::GetFromCommandFlag() {
+    return true;
+}
+
+void OverlayComp::SetFromCommandFlag(boolean flag) {
+}
+
 void OverlayComp::Configure(Editor*) { }
 
 OverlayComp* OverlayComp::TopComp() {
@@ -353,7 +367,7 @@ AttributeValue* OverlayComp::FindValue
     cerr << "breadth search not yet unsupported\n";
     return nil;
   } else if (up) {
-    cerr << "upward search not yet unsupported\n";
+    if (GetParent()) return ((OverlayComp*)GetParent())->FindValue(symid, last, breadth, down, up);
     return nil;
   } else if (last) {
     cerr << "search for last value not yet unsupported\n";
@@ -367,6 +381,7 @@ AttributeValue* OverlayComp::FindValue
 /*****************************************************************************/
 
 ParamList* OverlaysComp::_overlay_comps_params = nil;
+int OverlaysComp::_symid = -1;
 
 OverlaysComp::OverlaysComp (OverlayComp* parent) : OverlayComp(new Picture, parent) { 
     _comps = new UList;
@@ -604,6 +619,46 @@ void OverlaysComp::Interpret (Command* cmd) {
             cmd->GetClipboard()->Append(this);
         }
 
+    } else if (cmd->IsA(PUSH_CMD) || cmd->IsA(PULL_CMD)) {
+        Component* edComp = cmd->GetEditor()->GetComponent();
+
+        if (edComp == (Component*) this) {
+            Clipboard* cb = cmd->GetClipboard();
+            Iterator i;
+
+            if (cmd->IsA(PULL_CMD)) {
+                for (cb->First(i); !cb->Done(i); cb->Next(i)) {
+                    OverlayComp* comp = (OverlayComp*)cb->GetComp(i);
+                  Iterator j;
+                  SetComp(comp, j);
+                  Next(j);
+                    StorePosition(comp, cmd);
+                  if (!Done(j)) {
+                    Remove(comp);
+                    InsertAfter(j, comp);
+                  }
+                }
+
+            } else {
+                for (cb->Last(i); !cb->Done(i); cb->Prev(i)) {
+                    OverlayComp* comp = (OverlayComp*) cb->GetComp(i);
+                  Iterator j;
+                  SetComp(comp, j);
+                  Prev(j);
+                    StorePosition(comp, cmd);
+                  if (!Done(j)) {
+                    Remove(comp);
+                    InsertBefore(j, comp);
+                  }
+                }
+            }
+            Notify();
+            unidraw->Update();
+
+        } else {
+            OverlayComp::Interpret(cmd);
+        }
+
     } else if (cmd->IsA(FRONT_CMD) || cmd->IsA(BACK_CMD)) {
         Component* edComp = cmd->GetEditor()->GetComponent();
 
@@ -874,14 +929,22 @@ void OverlaysComp::InsertBefore (Iterator i, GraphicComp* comp) {
 void OverlaysComp::InsertAfter (Iterator i, GraphicComp* comp) {
     Graphic* g = comp->GetGraphic();
     Graphic* parent;
-    
-    Elem(i)->Prepend(new UList(comp));
+
+    if (Elem(i))
+      Elem(i)->Prepend(new UList(comp));
+    else {
+      cerr << "OverlaysComp::InsertAfter -- Iterator has nil value\n";
+      return;
+    }
     
     if (g != nil) {
         Iterator j;
         parent = GetGraphic();
-        parent->SetGraphic(GetComp(i)->GetGraphic(), j);
-        parent->InsertAfter(j, g);
+	GraphicComp* comp = GetComp(i);
+	if (comp) {
+	  parent->SetGraphic(comp->GetGraphic(), j);
+	  parent->InsertAfter(j, g);
+	}
     }
     SetParent(comp, this);
 }
@@ -1085,8 +1148,13 @@ AttributeValue* OverlaysComp::FindValue
     cerr << "breadth search not yet unsupported\n";
     return nil;
   } else if (up) {
-    cerr << "upward search not yet unsupported\n";
-    return nil;
+    if (AttributeList* al = attrlist()) {
+      AttributeValue* av = al->find(symid);
+      if (av) return av;
+    } 
+    return GetParent() 
+      ? ((OverlayComp*)GetParent())->FindValue(symid, last, breadth, down, up) 
+      : nil;
   } else if (last) {
     cerr << "search for last value not yet unsupported\n";
     return nil;
@@ -1107,6 +1175,7 @@ AttributeValue* OverlaysComp::FindValue
 /*****************************************************************************/
 
 ParamList* OverlayIdrawComp::_overlay_idraw_params = nil;
+int OverlayIdrawComp::_symid = -1;
 
 OverlayIdrawComp::OverlayIdrawComp (const char* pathname, OverlayComp* parent)
 : OverlaysComp(parent) {
