@@ -52,7 +52,9 @@ protected:
     void GrowParamList(ParamList*);
     static ParamList* _ovpolygon_params;
 
-friend OverlaysScript;
+friend class OverlaysScript;
+
+    CLASS_SYMID("PolygonComp");
 };
 
 //: graphical view of PolygonOvComp.
