@@ -50,6 +50,7 @@
 /*****************************************************************************/
 
 ParamList* StencilOvComp::_ovstencil_params = nil;
+int StencilOvComp::_symid = -1;
 
 static const int no_mask = 0;
 static const int mask_equals_image = 1;
@@ -57,6 +58,7 @@ static const int valid_mask = 2;
 
 /*****************************************************************************/
 
+
 ClassId StencilOvComp::GetClassId () { return OVSTENCIL_COMP; }
 
 boolean StencilOvComp::IsA (ClassId id) {
@@ -167,7 +169,7 @@ boolean StencilPS::Definition (ostream& out) {
     UStencil* stencil = (UStencil*) GetGraphicComp()->GetGraphic();
     Bitmap* image, *mask;
     stencil->GetOriginal(image, mask);
-    char* tag = (image == mask) ? "SSten" : "FSten";
+    const char* tag = (image == mask) ? "SSten" : "FSten";
     Coord w = image->Width();
     Coord h = image->Height();
 
@@ -286,7 +288,8 @@ int StencilScript::ReadStencil (istream& in, void* addr1, void* addr2, void* add
     
     else if (urlflag || 
 	     strcmp(creator, "JPEG") == 0 || 
-	     strcmp(creator, "GIF")==0) {
+	     strcmp(creator, "GIF") == 0 || 
+	     strcmp(creator, "PNG")==0) {
         OvImportCmd importcmd((Editor*)nil);
 	OverlayComp* tempcomp = (OverlayComp*)importcmd.Import(pathname);
 	if (tempcomp && tempcomp->IsA(OVSTENCIL_COMP)) {
