@@ -38,7 +38,7 @@
 #include <IV-2_6/_enter.h>
 
 class Vertices;
-class istream;
+#include <iosfwd>
 
 //: clone of VerticesComp derived from OverlayComp.
 // base class for all multi-point components.
@@ -55,6 +55,8 @@ public:
 protected:
     VerticesOvComp(Vertices* = nil, OverlayComp* parent = nil);
     VerticesOvComp(istream&, OverlayComp* parent = nil);
+
+    CLASS_SYMID("VerticesComp"); 
 };
 
 //: graphical view of VerticesOvComp.
