@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1994-1998 Vectaport Inc.
+ * Copyright (C) 2002 Scott E. Johnston
+ * Copyright (C) 1994-1998 Vectaport Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided
@@ -26,6 +27,7 @@
 #include <OverlayUnidraw/scriptview.h>
 #include <OverlayUnidraw/ovcatalog.h>
 #include <OverlayUnidraw/ovclasses.h>
+#include <OverlayUnidraw/ovexport.h>
 
 #include <ComTerp/parser.h>
 
@@ -56,6 +58,8 @@
 #include <InterViews/textbuffer.h>
 #include <InterViews/transformer.h>
 
+#include <OS/math.h>
+
 #include <ctype.h>
 #include <stream.h>
 #include <stdio.h>
@@ -63,6 +67,9 @@
 
 /*****************************************************************************/
 
+boolean OverlayScript::_ptlist_parens = true;
+boolean OverlayScript::_svg_format = false;
+
 ClassId OverlayScript::GetClassId () { return OVERLAY_SCRIPT; }
 
 boolean OverlayScript::IsA (ClassId id) {
@@ -77,8 +84,10 @@ OverlayScript::~OverlayScript () {
 }
 
 void OverlayScript::FillBg (ostream& out) {
-    int filled = GetOverlayComp()->GetGraphic()->BgFilled();
+  int filled = GetOverlayComp()->GetGraphic()->BgFilled();
+  if (!svg_format()) {
     out << " :fillbg " << filled;
+  }
 }
 
 void OverlayScript::Brush (ostream& out) {
@@ -86,30 +95,103 @@ void OverlayScript::Brush (ostream& out) {
 
     if (brush != nil) {
 
-	if (brush->None()) {
-	    out << " :nonebr";
-
-	} else {
-	    out << " :brush ";
-	    int p = brush->GetLinePattern();
-	    out << p << ",";
-
-	    float w = brush->width();
-	    out << w;
+      if (brush->None()) {
+	
+	if (!svg_format()) {
+	  out << " :nonebr";
 	}
+	
+      } else {
+	int p = brush->GetLinePattern();
+	float w = brush->width();
+
+	if (!svg_format()) {
+	  out << " :brush ";
+	  out << p << ",";
+	  out << w;
+	} else {
+	  out << "stroke-width: " << w << "; ";
+
+	  if (p != 0xffff) {
+	    out << "stroke-dasharray: ";
+	    
+	    /* reverse dash bit field */
+	    int reverse_p = 0;
+	    for (int i=0; i<16; i++) {
+	      reverse_p <<= 1;
+	      reverse_p |= p & 0x1;
+	      p >>= 1;
+	    }
+	    
+	    /* output count of on/off runs */
+	    boolean lastbit=1;
+	    int dashlen = 0;
+	    for (int i=0; i<16; i++) {
+	      boolean bit = reverse_p & 1;
+	      if (lastbit==bit) dashlen++;
+	      if (lastbit != bit || i==15)  {
+		out << dashlen;
+		if (i != 15 ) out << ", ";
+		dashlen=1;
+	      }
+	      lastbit = bit;
+	      reverse_p >>= 1;
+	    }
+	    out << "; ";
+	  }
+	}
+	
+      }
     }
 }
 
+void OverlayScript::Colors (ostream& out) {
+  if (!svg_format()) {
+    FgColor(out);
+    BgColor(out);
+  } else {
+
+    PSColor* fgcolor = (PSColor*) GetOverlayComp()->GetGraphic()->GetFgColor();
+    PSColor* bgcolor = (PSColor*) GetOverlayComp()->GetGraphic()->GetBgColor();
+
+    ColorIntensity fr, fg, fb;
+    fgcolor->GetIntensities(fr, fg, fb);
+
+    out << "stroke: rgb(" 
+	<< (int)(fr*100) << "%,"
+	<< (int)(fg*100) << "%,"
+	<< (int)(fb*100) << "%); ";
+    
+    ColorIntensity br, bg, bb;
+    bgcolor->GetIntensities(br, bg, bb);
+
+    PSPattern* pat = (PSPattern*) GetOverlayComp()->GetGraphic()->GetPattern();
+    if (pat != nil && !pat->None() && 
+	pat->GetGrayLevel()>=0.0 && pat->GetGrayLevel()<=1.0) { 
+      
+      float fp = 100.0*(1.0-pat->GetGrayLevel());
+      float bp = 100.0-fp;
+      out << "fill: rgb(" 
+	  << (int)(fr*fp+br*bp) << "%,"
+	  << (int)(fg*fp+bg*bp) << "%,"
+	  << (int)(fb*fp+bb*bp) << "%); ";
+    }
+  }
+  
+}
+
 void OverlayScript::FgColor (ostream& out) {
     PSColor* fgcolor = (PSColor*) GetOverlayComp()->GetGraphic()->GetFgColor();
 
     if (fgcolor != nil) {
-	const char* name = fgcolor->GetName();
-	out << " :fgcolor \"" << name << "\"";
+      
+      const char* name = fgcolor->GetName();
+      ColorIntensity r, g, b;
+      fgcolor->GetIntensities(r, g, b);
+
+      out << " :fgcolor \"" << name << "\"";
+      out << "," << r << "," << g << "," << b;
 
-	ColorIntensity r, g, b;
-	fgcolor->GetIntensities(r, g, b);
-	out << "," << r << "," << g << "," << b;
     }
 }
 
@@ -117,24 +199,31 @@ void OverlayScript::BgColor (ostream& out) {
     PSColor* bgcolor = (PSColor*) GetOverlayComp()->GetGraphic()->GetBgColor();
 
     if (bgcolor != nil) {
-	const char* name = bgcolor->GetName();
-	out << " :bgcolor \"" << name << "\"";
 
-	ColorIntensity r, g, b;
-	bgcolor->GetIntensities(r, g, b);
+      const char* name = bgcolor->GetName();
+      ColorIntensity r, g, b;
+      bgcolor->GetIntensities(r, g, b);
+
+	out << " :bgcolor \"" << name << "\"";
 	out << "," << r << "," << g << "," << b;
+
     }
 }
 
 void OverlayScript::Font (ostream& out) {
     PSFont* font = (PSFont*) GetOverlayComp()->GetGraphic()->GetFont();
     if (font != nil) {
+
 	const char* name = font->GetName();
-	out << " :font \"" << name << "\"" << ",";
 	const char* pf = font->GetPrintFont();
-	out << "\"" << pf << "\"" << ",";
 	const char* ps = font->GetPrintSize();
-	out << ps;
+
+	if (!svg_format()) {
+	  out << " :font \"" << name << "\"" << ",";
+	  out << "\"" << pf << "\"" << ",";
+	  out << ps;
+	}
+	
     }
 }
 
@@ -142,46 +231,53 @@ void OverlayScript::Pattern (ostream& out) {
     PSPattern* pat = (PSPattern*) GetOverlayComp()->GetGraphic()->GetPattern();
 
     if (pat != nil) {
+      
+      if (pat->None()) {
 
+	if (!svg_format())
+	  out << " :nonepat";
+	else 
+	  out << "fill: none;";
 
-	if (pat->None()) {
-	    out << " :nonepat";
-	} else if (pat->GetSize() > 0) {
-	    const int* data = pat->GetData();
-	    int size = pat->GetSize();
-	    char buf[BUFSIZ];
-	    out << " :pattern ";
+      } else if (pat->GetSize() > 0) {
+	
+	const int* data = pat->GetData();
+	int size = pat->GetSize();
+	char buf[BUFSIZ];
 
-	    if (size <= 8) {
-		for (int i = 0; i < 8; i++) {
-		    sprintf(buf, "0x%02x", data[i] & 0xff);
-		    out << buf;
-		    if (i < 7 ) out << ",";
-		}
+	if (!svg_format()) {
 
-	    } else {
-		for (int i = 0; i < patternHeight; i++) {
-		    sprintf(buf, "0x%0*x", patternWidth/4, data[i]);
-		    if (i != patternHeight - 1) {
-			out << buf << ",";
-		    } else {
-			out << buf;
-		    }
-		}
+	  out << " :pattern ";
+	  
+	  if (size <= 8) {
+	    for (int i = 0; i < 8; i++) {
+	      sprintf(buf, "0x%02x", data[i] & 0xff);
+	      out << buf;
+	      if (i < 7 ) out << ",";
 	    }
-
-	} else {
-	    float graylevel = pat->GetGrayLevel();
-	    out << " :graypat " << graylevel;
+	    
+	  } else {
+	    for (int i = 0; i < patternHeight; i++) {
+	      sprintf(buf, "0x%0*x", patternWidth/4, data[i]);
+	      if (i != patternHeight - 1) {
+		out << buf << ",";
+	      } else {
+		out << buf;
+	      }
+	    }
+	  }
 	}
+	
+      } else {
+	float graylevel = pat->GetGrayLevel();
+	if (!svg_format()) {
+	  out << " :graypat " << graylevel;
+	}
+      }
     }
 }
 
-void OverlayScript::Transformation (ostream& out) {
-    Transformation(out, "transform");
-}
-
-void OverlayScript::Transformation (ostream& out, char* keyword, Graphic* gr) {
+void OverlayScript::Transformation(ostream& out, char* keyword, Graphic* gr) {
     Transformer* t = gr ? gr->GetTransformer() : GetOverlayComp()->GetGraphic()->GetTransformer();
     Transformer identity;
 
@@ -196,14 +292,36 @@ void OverlayScript::Transformation (ostream& out, char* keyword, Graphic* gr) {
     }
 }
 
+void OverlayScript::Transformation (ostream& out) {
+    Graphic *gr = GetOverlayComp()->GetGraphic();
+    Transformer* t = gr ? gr->GetTransformer() : nil;
+    Transformer identity;
+
+    if (t != nil && *t != identity) {
+      float a00, a01, a10, a11, a20, a21;
+      t->GetEntries(a00, a01, a10, a11, a20, a21);
+      if (!svg_format()) {
+	out << " :transform ";
+	out << a00 << "," << a01 << "," << a10 << ",";
+	out << a11 << "," << a20 << "," << a21;
+      } else {
+	out << "transform=\"matrix(";
+	out << a00 << " " << a01 << " " << a10 << " ";
+	out << a11 << " " << a20 << " " << a21 << ")\"";
+      }
+    }
+}
+
 void OverlayScript::Annotation (ostream& out) {
     OverlayComp* comp = GetOverlayComp();
     const char* anno = comp->GetAnnotation();
     if (!anno) 
         return;
-    out << " :annotation " << "\n";
-    int indent = Indent(out);
-    ParamList::output_text(out, anno, indent);
+    if (!svg_format()) {
+      out << " :annotation " << "\n";
+      int indent = Indent(out);
+      ParamList::output_text(out, anno, indent);
+    }
 }
 
 OverlayScript* OverlayScript::CreateOverlayScript (OverlayComp* comp) {
@@ -258,6 +376,17 @@ boolean OverlayScript::skip_comp(istream& in) {
   return true;
 }
   
+boolean OverlayScript::svg_format() {
+    boolean format = OverlayScript::_svg_format;
+    Command* cmd = GetCommand();
+    if (cmd) {
+      if (GetCommand()->IsA(OV_EXPORT_CMD))
+	format = ((OvExportCmd*)GetCommand())->svg_format();
+    }
+    return format;
+}
+
+void OverlayScript::svg_format(boolean flag) { _svg_format = flag; }
 
 /*****************************************************************************/
 
@@ -361,9 +490,9 @@ int OverlayScript::ReadFgColor (istream& in, void* addr1, void* addr2, void* add
 	    return -1;
         }                
 	else {
-	    int ir = round(r * float(0xffff));
-	    int ig = round(g * float(0xffff));
-	    int ib = round(b * float(0xffff));
+	    int ir = Math::round(r * float(0xffff));
+	    int ig = Math::round(g * float(0xffff));
+	    int ib = Math::round(b * float(0xffff));
 	    PSColor* fgcolor = OverlayCatalog::Instance()->FindColor(name_arg ? name : "no_name", ir, ig, ib);
 	    gs->SetColors(fgcolor, gs->GetBgColor());
             return 0;
@@ -405,9 +534,9 @@ int OverlayScript::ReadBgColor (istream& in, void* addr1, void* addr2, void* add
 	    return -1;
         }                
 	else {
-	    int ir = round(r * float(0xffff));
-	    int ig = round(g * float(0xffff));
-	    int ib = round(b * float(0xffff));
+	    int ir = Math::round(r * float(0xffff));
+	    int ig = Math::round(g * float(0xffff));
+	    int ib = Math::round(b * float(0xffff));
 
 	    PSColor* bgcolor = OverlayCatalog::Instance()->FindColor(name_arg ? name : "no_name", ir, ig, ib);
 	    gs->SetColors(gs->GetFgColor(), bgcolor);
@@ -524,7 +653,8 @@ int OverlayScript::ReadTransform (istream& in, void* addr1, void* addr2, void* a
     }
     else {
         Transformer* t = new Transformer(a00, a01, a10, a11, a20, a21);
-        gs->SetTransformer(t);
+        if (gs) gs->SetTransformer(t);
+        else fprintf(stderr, "OverlayScript::ReadTransform:  no graphic for transformer\n");
 	Unref(t);
         return 0;
     }
@@ -551,15 +681,17 @@ int OverlayScript::ReadOther(istream& in, void* addr1, void* addr2, void* addr3,
     AttributeValue* val;
 
     do {
-    if (in.peek() == '\"') {
+    char ch;
+    if ((ch=in.peek()) == '\"') {
 	sbuf[0] = '\"';
 	ParamList::parse_string(in, sbuf+1, SBUFSIZE-1);
 	strcat(sbuf, "\"\n");
     }
-    else {
+    else if (ch!=')') {
 	ParamList::parse_token(in, sbuf, SBUFSIZE, " \t\n,");
 	strcat(sbuf, "\n");
-    }
+    } else
+      strcpy(sbuf, "1\n");
 
     if (!in.good() && attrlist && keyword) {
         return -1;
@@ -645,8 +777,7 @@ boolean OverlayScript::EmitGS(ostream& out, Clipboard* cb, boolean prevout) {
 	out << "    gs(";
 	FillBg(out);
 	Brush(out);
-	FgColor(out);
-	BgColor(out);
+	Colors(out);
 	Font(out);
 	Pattern(out);
 	out << ")";
@@ -704,7 +835,7 @@ Iterator OverlayScript::MatchedPic(Clipboard* cb, int& count) {
 boolean OverlayScript::EmitPts(ostream& out, Clipboard* cb, boolean prevout) {
     if (GetGraphicComp()->IsA(OVVERTICES_COMP) && MatchedPts(cb)<0) {
 	MultiLineObj* pts = ((Vertices*)GetGraphicComp()->GetGraphic())->GetOriginal();
-	if (pts->count()>0) {
+	if (pts && pts->count()>0) {
 	  if (prevout) 
 	    out << ",\n    ";
 	  else
@@ -738,14 +869,15 @@ boolean OverlayScript::EmitPic(ostream& out, Clipboard* cb1, Clipboard* cb2, boo
 void OverlayScript::MinGS (ostream& out) {
     if (!DefaultGS()) {
 	Clipboard* cb = GetGSList();
-	if (cb) 
+	if (cb && !svg_format()) 
 	    out << " :gs " << MatchedGS(cb);
 	else {
+	    if (svg_format()) out << "style=\"";
 	    FillBg(out);
 	    Brush(out);
-	    FgColor(out);
-	    BgColor(out);
+	    Colors(out);
 	    Pattern(out);
+	    if (svg_format()) out << "\" ";
 	}
     }
     Transformation(out);
@@ -759,8 +891,7 @@ void OverlayScript::FullGS (ostream& out) {
 	else {
 	    FillBg(out);
 	    Brush(out);
-	    FgColor(out);
-	    BgColor(out);
+	    Colors(out);
 	    Font(out);
 	    Pattern(out);
 	} 
@@ -788,8 +919,7 @@ void OverlayScript::StencilGS (ostream& out) {
 	if (cb) 
 	    out << " :gs " << MatchedGS(cb);
 	else {
-	    FgColor(out);
-	    BgColor(out);
+	    Colors(out);
 	} 
     }
     Transformation(out);
@@ -1157,6 +1287,9 @@ boolean OverlayIdrawScript::IsA (ClassId id) {
 }
 
 boolean OverlayIdrawScript::Emit (ostream& out) {
+    if (svg_format()) 
+      return EmitSvg(out);
+
     out << "drawtool(";
 
     /* make list and output unique point lists */
@@ -1248,3 +1381,47 @@ boolean OverlayIdrawScript::GetByPathnameFlag() {
     return _by_pathname;
 }
 
+boolean OverlayIdrawScript::EmitSvg (ostream& out) {
+  out << "<?xml version=\"1.0\"?>\n";
+  out << "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.0/EN\"\n";
+  out << "    \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">\n";
+
+  Coord l, b, r, t;
+  GetBox(l, b, r, t);
+
+  Coord w = r - l;
+  Coord h = t - b;
+
+  out << "<svg width=\"" << w << "\" height=\"" << h << "\" >\n";
+  out << "<title>ivtools drawing</title>\n";
+  out << "<desc>ivtools drawing</desc>\n";
+
+  out << "<g transform=\"matrix(1 0 0 -1 " << -l << " " << t << ")\" >\n";
+
+  /* output graphic components */
+  boolean status = true;
+  Iterator i;
+  First(i);
+
+  static int readonly_symval = symbol_add("readonly");
+  for (; status && !Done(i); ) {
+    OverlayScript* ev = (OverlayScript*)GetView(i);
+    boolean readonly = false;
+    AttributeList *al;
+    if (al = ev->GetOverlayComp()->attrlist()) {
+      AttributeValue* av = al->find(readonly_symval);
+      if (av) readonly = av->is_true();
+    }
+    if (!readonly) {
+      Indent(out);
+      status = ev->Definition(out);
+    }
+    Next(i);
+  }
+
+  out << "</g>\n";
+  out << "</svg>\n";
+  return status;
+}
+
+
