@@ -51,20 +51,8 @@ const unsigned VERSION =2;
 
 #endif
 
-#if defined(BSD) || defined(hpux) || defined(linux) || defined(__sun) || defined(__alpha)
-
-#ifdef hpux
-#include <time.h>
-#else
-#include <sys/time.h>
-#if defined(__DECCXX) || (defined(__sun) && !defined(__svr4__))
-extern "C" {
-    void gettimeofday(struct timeval *tp, struct timezone *tzp);
-}
-#endif
-
-#endif
 
+/************************ end of edits ************************************************/
 static long TIME_ZONE;          /* seconds west of GMT */
 static int DST_OBSERVED;        /* flags U.S. daylight saving time observed */
 
@@ -81,9 +69,6 @@ static void inittimezone() {
 #define VIRTUAL_BASE_CLASSES Object::desc()
 
 
-#endif
-
-
 static const unsigned long seconds_in_day = 24*60*60;
 static const Date refDate(0L);
 static const Date maxDate(31,"Dec",2036);
@@ -142,7 +127,7 @@ Time::Time(const Date& date, hourTy h, minuteTy m, secondTy s, boolean dst)
 	sec += TIME_ZONE;				// adjust to GMT 
 }
 
-Time::operator Date() const
+Date Time::date() const
 /*
 	Convert a Time to a local Date
 */
@@ -237,7 +222,7 @@ Time Time::localTime() const
 	Adjusts this GM Time for local time zone and Daylight Savings Time.
 */
 {
-	Time &local_time = *new Time(sec-TIME_ZONE);
+	Time local_time(sec-TIME_ZONE);
 	if (local_time.isDST()) local_time.sec += 3600;
 	return local_time;
 }
@@ -294,7 +279,7 @@ Time Time::min(const Time& t) const
 void Time::printOn(ostream& strm) const
 {
 	register unsigned hh = hour();
-	Date(*this).printOn(strm);
+	this->date().printOn(strm);
  	strm << ' ' << ((hh <= 12) ? hh : hh-12) << ':';
  	strm << setfill('0') << setw(2) << minute() << ':';
  	strm << setfill('0') << setw(2) << second() << ' ';
