@@ -1,22 +1,23 @@
 /*
+ * Copyright (c) 1994 Vectaport Inc.
  * Copyright (c) 1990, 1991 Stanford University
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided
+ * Permission to use, copy, modify, distribute, and sell this software and
+ * its documentation for any purpose is hereby granted without fee, provided
  * that the above copyright notice appear in all copies and that both that
  * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of Stanford not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  Stanford makes no representations about
- * the suitability of this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
+ * documentation, and that the names of the copyright holders not be used in
+ * advertising or publicity pertaining to distribution of the software
+ * without specific, written prior permission.  The copyright holders make
+ * no representations about the suitability of this software for any purpose.
+ * It is provided "as is" without express or implied warranty.
  *
- * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
- * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
- * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
+ * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
+ * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
+ * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
+ * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
+ * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
+ * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
@@ -28,6 +29,7 @@
 #include <Unidraw/Graphic/util.h>
 #include <Unidraw/ulist.h>
 
+#include <OS/math.h>
 #include <OS/memory.h>
 
 #include <IV-2_6/_enter.h>
@@ -240,12 +242,12 @@ void MultiLineObj::AddLine (double x0, double y0, double x1, double y1) {
 	GrowBuf();
     } 
     if (mlcount == 0) {
-	mlx[mlcount] = round(x0);
-	mly[mlcount] = round(y0);
+	mlx[mlcount] = Math::round(x0);
+	mly[mlcount] = Math::round(y0);
 	++mlcount;
     }
-    mlx[mlcount] = round(x1);
-    mly[mlcount] = round(y1);
+    mlx[mlcount] = Math::round(x1);
+    mly[mlcount] = Math::round(y1);
     ++mlcount;
 }
 
