@@ -1,11 +1,13 @@
 #ifndef ivtools_signal_h
 #define ivtools_signal_h
 
-#include_next <signal.h>
 
 #if defined(__cplusplus)
+#include_next <signal.h>
 #undef NULL
 #define NULL 0
+#else
+#include </usr/include/signal.h>
 #endif
 
 #endif
